Package: BATSS
Title: Bayesian Adaptive Trial Simulator Software (BATSS) for
        Generalised Linear Models
Version: 1.2.1
Authors@R: c(
  person("Dominique-Laurent", "Couturier", role = c("aut", "cre"), email = "dominique.couturier@mrc-bsu.cam.ac.uk", comment = c(ORCID = "0000-0001-5774-5036")),
  person("Liz", "Ryan", role = "aut", comment = c(ORCID = "0000-0001-9367-4204")),
  person("Rainer", "Puhr", role = "aut"),
  person("Thomas", "Jaki", role = "aut", comment = c(ORCID = "0000-0002-1096-188X")),
  person("Stephane", "Heritier", role = "aut", comment = c(ORCID = "0000-0002-3640-079X")))
Description: Defines operating characteristics of Bayesian Adaptive Trials considering a generalised linear model response via Monte Carlo simulations of Bayesian GLM fitted via integrated Laplace approximations (INLA). 
URL: https://batss-stable.github.io/BATSS/
License: GPL-2
Encoding: UTF-8
Suggests: INLA
Imports: parallel, methods, stats, grDevices, abind, plyr, rlang,
        R.utils, cli, sm
Additional_repositories: https://inla.r-inla-download.org/R/testing
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-08-06 16:14:35 UTC; research
Author: Dominique-Laurent Couturier [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-5774-5036>),
  Liz Ryan [aut] (ORCID: <https://orcid.org/0000-0001-9367-4204>),
  Rainer Puhr [aut],
  Thomas Jaki [aut] (ORCID: <https://orcid.org/0000-0002-1096-188X>),
  Stephane Heritier [aut] (ORCID:
    <https://orcid.org/0000-0002-3640-079X>)
Maintainer: Dominique-Laurent Couturier <dominique.couturier@mrc-bsu.cam.ac.uk>
Repository: CRAN
Date/Publication: 2026-09-01 12:30:27 UTC
Built: R 4.5.2; ; 2026-09-01 14:20:54 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,batss)
S3method(print,batss)
S3method(print,summary.batss)
S3method(summary,batss)
export(RAR.optimal)
export(RAR.trippa)
export(alloc.balanced)
export(alloc.simple)
export(batss.combine)
export(batss.glm)
export(eff.arm.infofract)
export(eff.arm.simple)
export(eff.trial.all)
export(eff.trial.any)
export(fut.arm.simple)
export(fut.trial.all)
import(R.utils)
import(abind)
import(grDevices)
import(graphics)
import(methods)
import(parallel)
import(plyr)
import(stats)
importFrom(cli,cli_h1)
importFrom(cli,cli_h2)
importFrom(cli,cli_h3)
importFrom(grDevices,dev.new)
importFrom(grDevices,devAskNewPage)
importFrom(grDevices,gray)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,matplot)
importFrom(graphics,matpoints)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(methods,is)
importFrom(plyr,is.formula)
importFrom(rlang,dots_list)
importFrom(stats,dnorm)
importFrom(stats,pnorm)
importFrom(stats,qbinom)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,uniroot)
importFrom(stats,var)
importFrom(utils,packageVersion)

# estimate matrix
batss.res.e = function(trial_r,id.target){       
    out = array(unlist(lapply(trial_r,function(x){
        type = rep(NA,nrow(x$target))
        for(i in 1:length(type)){
            eff = x$target$efficacy[i]&!is.na(x$target$efficacy[i])
            fut = x$target$futility[i]&!is.na(x$target$futility[i])
            type[i] = ifelse(eff&fut,3,ifelse(eff,1,ifelse(fut,2,0)))
        }
        c(x$target$look,type,x$target$mid)
        })),
        dim=c(nrow(id.target),3,length(trial_r)),
        dimnames=list(id.target$id,c("look","type","mid")))
    out[,"type",][is.na(out[,"type",])] = 0
    out
    }
# target per parameter
batss.res.tp = function(estimate,id.target){
    out = id.target[,c("pos","id","alternative","group")] 
    out$efficacy = apply(estimate[,"type",,drop=FALSE]==1,1,mean)
    out$futility = apply(estimate[,"type",,drop=FALSE]==2,1,mean)
    out$both     = apply(estimate[,"type",,drop=FALSE]==3,1,mean) 
    colnames(out)[1] = ""    
    out
    }
# target global   
batss.res.tg = function(estimate,id.target){
    out = data.frame(pos=1:2,id=c("At least one","All"),
                     alternative="",group="",efficacy=NA,futility=NA,both=NA)
    out$efficacy[1] = mean(apply(estimate[,"type",,drop=FALSE]==1,3,sum)>0)
    out$futility[1] = mean(apply(estimate[,"type",,drop=FALSE]==2,3,sum)>0)
    out$both[1]     = mean(apply(estimate[,"type",,drop=FALSE]==3,3,sum)>0)
    out$efficacy[2] = mean(apply(estimate[,"type",,drop=FALSE]==1,3,all)>0)
    out$futility[2] = mean(apply(estimate[,"type",,drop=FALSE]==2,3,all)>0)
    out$both[2]     = mean(apply(estimate[,"type",,drop=FALSE]==3,3,all)>0)
    colnames(out)[1] = ""    
    out
    }  
# efficacy per target parameter
batss.res.ep = function(estimate,id.target,n.look){
    out = id.target[,c("pos","id","alternative","group")]
    out$early   = apply((estimate[,"type",,drop=FALSE]==1)*(estimate[,"look",,drop=FALSE]<n.look),1,mean)
    out$last    = apply((estimate[,"type",,drop=FALSE]==1)*(estimate[,"look",,drop=FALSE]==n.look),1,mean)
    out$overall = apply((estimate[,"type",,drop=FALSE]==1),1,mean)
    colnames(out)[1] = ""    
    out
    }
# efficacy global
batss.res.eg = function(estimate,id.target,n.look){
    out = data.frame(pos=1:2,id=c("At least one","All"),
                     alternative="",group="",early=NA,last=NA,overall=NA)
    # at least one
    out$early[1] = mean(apply(estimate[,"type",,drop=FALSE]==1&(estimate[,"look",,drop=FALSE]<n.look),3,sum)>0)
    out$last[1]  = mean(apply(estimate[,"type",,drop=FALSE]==1&(estimate[,"look",,drop=FALSE]==n.look),3,sum)>0)
    out$overall[1]  = mean(apply(estimate[,"type",,drop=FALSE]==1,3,sum)>0)
    # all
    out$early[2] = mean(apply(estimate[,"type",,drop=FALSE]==1&(estimate[,"look",,drop=FALSE]<n.look),3,all)>0)
    out$last[2]  = mean(apply(estimate[,"type",,drop=FALSE]==1&(estimate[,"look",,drop=FALSE]==n.look),3,all)>0)
    out$overall[2]  = mean(apply(estimate[,"type",,drop=FALSE]==1,3,all)>0)
    # out
    colnames(out)[1] = ""    
    out
    }  
# futility per target parameter
batss.res.fp = function(estimate,id.target,n.look){
    out = id.target[,c("pos","id","alternative","group")]
    out$early   = apply((estimate[,"type",,drop=FALSE]==2)*(estimate[,"look",,drop=FALSE]<n.look),1,mean)
    out$last    = apply((estimate[,"type",,drop=FALSE]==2)*(estimate[,"look",,drop=FALSE]==n.look),1,mean)
    out$overall = apply((estimate[,"type",,drop=FALSE]==2),1,mean)
    colnames(out)[1] = ""    
    out
    }
# efficacy global
batss.res.fg = function(estimate,id.target,n.look){
    out = data.frame(pos=1:2,id=c("At least one","All"),
                     alternative="",group="",early=NA,last=NA,overall=NA)
    # at least one
    out$early[1] = mean(apply(estimate[,"type",,drop=FALSE]==2&(estimate[,"look",,drop=FALSE]<n.look),3,sum)>0)
    out$last[1]  = mean(apply(estimate[,"type",,drop=FALSE]==2&(estimate[,"look",,drop=FALSE]==n.look),3,sum)>0)
    out$overall[1]  = mean(apply(estimate[,"type",,drop=FALSE]==2,3,sum)>0)
    # all
    out$early[2] = mean(apply(estimate[,"type",,drop=FALSE]==2&(estimate[,"look",,drop=FALSE]<n.look),3,all)>0)
    out$last[2]  = mean(apply(estimate[,"type",,drop=FALSE]==2&(estimate[,"look",,drop=FALSE]==n.look),3,all)>0)
    out$overall[2]  = mean(apply(estimate[,"type",,drop=FALSE]==2,3,all)>0)
    # out
    colnames(out)[1] = ""    
    out
    }  
batss.res.s1 = function(trial_r,group,type,early){
    size = as.data.frame(matrix(unlist(lapply(trial_r,function(x,group){
        x$look[max(x$target$look,na.rm=TRUE),paste0("n(",group,")")]
        },group=group)),byrow=TRUE,ncol=length(group),
        dimnames=list(names(trial_r),group)))
    cbind(size,type,early)    
    }
batss.res.s2 = function(sample,target){
    tablew = table(sample$type)
    tablew = tablew[order(tablew,decreasing=TRUE)]
    out    = data.frame(pos=1:length(tablew),id=names(tablew),
                        overall=c(tablew)/sum(tablew),early=NA)
    early  = round(tapply(sample$early,sample$type,mean),2)
    out[names(early),"early"] = early
    for(i in 1:length(target)){
        out = cbind(out,as.numeric(substr(out$id,i,i)))
        colnames(out)[ncol(out)] = target[i]
        }
    out = cbind(out[,-c(3:4)],out[,3:4])    
    colnames(out)[1] = ""
    out
    }
## useful functions (other)
batss.trial = function(int,data,model,link,family,beta,prob0,
                     RAR,RAR.control,
                     eff.arm,eff.trial,
                     eff.arm.control,eff.trial.control,
                     fut.arm,fut.trial,
                     fut.arm.control,fut.trial.control,
                     id.target,n.target,
                     id.look,n.look,
                     id.group,n.group,groupvar,
                     twodelta,delta.eff,delta.fut,delta.RAR,
                     var,var.control,id.var,n.var,
                     #linux.os=linux.os,
                     extended,...){
# int=2
                     
    # cat(paste0("\t start:",int,"\n"))
    set.seed((n.look+1)*int)  

    # generate data for initial panel
    n = m = N = prob = ref = target = ref = active = mu = posterior = NULL
    env = new.env()
    assign("m",id.look[1,"m"], envir = env)
    assign("n",id.look[1,"n"], envir = env)
    assign("prob",prob0 , envir = env)
    assign("var",var , envir = env)
    assign("var.control",var.control , envir = env)
    assign("N", id.look$n[n.look], envir = env)  
    assign("ref",id.group$reference, envir = env)
    
    #call functions from 'var'
    pos.col <- 2                                                                             # initialize the column indicator (starting at two, column one is the response)
    for(vw in 2:length(var)) {                                                               # cycle through all variables in 'var' - starting from 2 because the first variable here is the response
      tmp_nam <- names(var)[vw]                                                              # store current variable name
      args_ <- plyr::.(n=m,m=m,prob=prob)                                                    # set function arguments, these are preset in the trial function loop
      if (tmp_nam %in% names(var.control)) args_ <- c(args_, var.control[[tmp_nam]])         # add additional arguments if specified in 'var.control'
      tmp_var <- R.utils::doCall(var[[tmp_nam]], envir = env, args = args_)                  # call variable generating function
      if (!is.matrix(tmp_var)) {                                                             # check if the generated data is NOT a matrix 
        data[, pos.col] <- tmp_var                                                           # fill column in 'data' according to position indication
        pos.col <- pos.col+1                                                                 # increase position indicator
      } else {
        colnames(tmp_var) <- paste0(tmp_nam,1:dim(tmp_var)[2])                               # name columns of matrix 'name'1,'name'2, etc
        for (jj in 1:dim(tmp_var)[2]) {
          data[, pos.col] <- tmp_var[,jj]                                                    # cycle through columns and fill 'data' accordingly
          pos.col <- pos.col+1                                                               # keep track of position in 'data'
        }
      }
    }
    #X = model.matrix(as.formula(paste0("~",strsplit(model,"~")[[1]][2])),data=data)
    X <- model.matrix(model[-2], data = data)                                            
    #---
    XB = X%*%beta
    assign("mu",switch(link,
                       "identity" = XB,
                       "log" = exp(XB),
                       "logit" = INLA::inla.link.logit(XB, inverse=TRUE),
                       "probit" = INLA::inla.link.probit(XB, inverse=TRUE),
                       "robit" = INLA::inla.link.robit(XB, inverse=TRUE),
                       "cauchit" = INLA::inla.link.cauchit(XB, inverse=TRUE),
                       "loglog" = INLA::inla.link.loglog(XB, inverse=TRUE),
                       "cloglog" = INLA::inla.link.cloglog(XB, inverse=TRUE)),envir=env)
          
    tmp_nam <- names(var)[1] 
    args_ <- plyr::.(n=m,mu=mu)                                                             # create a quoted(!) list of available 'ingredients'
    if (!(identical(var[[1]],rbinom) || identical(var[[1]],rnbinom))) {
      names(args_)[1:2] <- formalArgs(var[[1]])[1:2]
    } else {
      if (identical(var[[1]],rbinom)) {
        names(args_)[1:2] <- c("n","prob")
      } else if (identical(var[[1]],rnbinom)) {
        names(args_)[1:2] <- c("n","mu")
      }
    }                                                                                       # rename the list objects to the names required by specified formula (NOTE: the order of the items is set, if a function requires a different order this will not work, clever rearranging may be needed)
    if (tmp_nam %in% names(var.control)) args_ <- c(args_, var.control[[tmp_nam]])          # add extra arguments if provided
    data[, id.var[1]] = R.utils::doCall(var[[1]], args = args_, envir = env)                # execute in 'env' environment with unused arguments allowed
    #---
    #cat("B")
    # prepare
    posterior.fun = function(inf,fit,delta){
        prob = INLA::inla.pmarginal(delta, fit$marginals.fixed[[unlist(inf[1])]])
        ifelse(inf[2]=="greater",1-prob,prob)
    }    
    mx.posterior_eff.lt = mx.posterior_fut.lt = matrix(NA,nrow=n.look,ncol=n.target,dimnames=list(id.look$id,id.target$id))
    if (!is.null(RAR))  mx.posterior_RAR.lt = mx.posterior_eff.lt
    mx.futility.lt = mx.efficacy.lt = matrix(FALSE,nrow=n.look,ncol=n.target,
                             dimnames=list(id.look$id,id.target$id))
    mx.rprob.lt = matrix(NA,nrow=n.look,ncol=n.group,
                         dimnames=list(id.look$id,id.group$id))   
    #cat("C")    
    
    dots <- rlang::dots_list(...,.named=TRUE) # dots=NULL
    if ("control.family" %in% names(dots)) {
      dots$control.family <- c(dots$control.family,list(control.link=list(model=link)))
    }

    # loop
    #if(INLA::inla.os.type()=="linux"&!is.na(linux.os)){
    #    INLA::inla.binary.install(os=linux.os,verbose=TRUE,md5.check=FALSE)
    #    }

    for(lw in 1:n.look){# lw=0; lw=lw+1
        # size
        #cat(.p("look:",lw,"\n"))
        temp = table(data[,groupvar])
        id.look[lw,names(temp)] = temp
        assign("n",temp, envir = env)
        assign("ref",id.group$reference, envir = env)
        #cat("D")
        # fit
        if ("control.family" %in% names(dots)) {
          # fit = inla(formula=model, data=data, family=family,
           #           verbose=FALSE,dots)     does nor  work like this
          fit = do.call(INLA::inla,c(list(formula=model, data=data, family=family,
                     verbose=FALSE),dots))
        } else {
          # fit = inla(formula=model, data=data, family=family,
          #            control.family=list(control.link=list(model=link)),
          #            verbose=FALSE,dots)
          fit = do.call(INLA::inla,c(list(formula=model, data=data, family=family,
                     control.family=list(control.link=list(model=link)),
                     verbose=FALSE),dots))
        }
        #cat("E")           
        # posteriors, efficacy and futility
        aw = id.target$active
        if (!is.null(eff.arm)) {
          mx.posterior_eff.lt[lw,aw] = apply(id.target[aw,c("id","alternative"),drop=FALSE],1,
                                           posterior.fun,fit=fit,delta=delta.eff[lw]) 
        } else {
          mx.posterior_eff.lt[lw,aw] = NA
        }
        if (twodelta || (is.null(eff.arm) && !is.null(fut.arm))){
            mx.posterior_fut.lt[lw,aw] = apply(id.target[aw,c("id","alternative"),drop=FALSE],1,
                                               posterior.fun,fit=fit,delta=delta.fut[lw])               
        }else{
            if (!is.null(fut.arm)) {
              mx.posterior_fut.lt[lw,aw] = mx.posterior_eff.lt[lw,aw]   
            } else {
              mx.posterior_fut.lt[lw,aw] = NA
            }
        }
        if (!is.null(RAR)) {
          mx.posterior_RAR.lt[lw,aw] = apply(id.target[aw,c("id","alternative"),drop=FALSE],1,
                                             posterior.fun,fit=fit,delta=delta.RAR[lw])
        }
        
        #cat("F")           
        # update mx.futility.lt and mx.efficacy.lt
        for(tw in 1:n.target){
            if(aw[tw]){
                # efficacy
                assign("posterior",mx.posterior_eff.lt[lw,tw], envir = env)
                    # assign("target",names(id.look[lw,names(temp)])==id.target[tw,"group"],envir = env)
                if (is.null(eff.arm) || is.na(delta.eff[lw])) {
                    mx.efficacy.lt[lw,tw] = FALSE
                  } else {
                    mx.efficacy.lt[lw, tw] = R.utils::doCall(eff.arm, args = c(plyr::.(posterior=posterior,n=n,N=N,target=target,ref=ref),eff.arm.control), envir = env)        #call function instead of parsing and evaluating string
                }
                # futility
                if(twodelta || (is.null(eff.arm) && !is.null(fut.arm))){
                    assign("posterior",mx.posterior_fut.lt[lw,tw], envir = env)
                    }
                if (is.null(fut.arm) || is.na(delta.fut[lw])) {
                    mx.futility.lt[lw,tw] = FALSE
                  } else {
                    mx.futility.lt[lw, tw] = R.utils::doCall(fut.arm, args = c(plyr::.(posterior=posterior,n=n,N=N,target=target,ref=ref),fut.arm.control), envir = env)        #call function instead of parsing and evaluating string
                  }
                #---
            }else{
                mx.efficacy.lt[lw,tw] = FALSE
                mx.futility.lt[lw,tw] = FALSE
            }
        }
        #cat("G")           
        eff.target = apply(mx.efficacy.lt[1:lw,,drop=FALSE],2,any)
        fut.target = apply(mx.futility.lt[1:lw,,drop=FALSE],2,any)

        if (!is.null(eff.arm)) {
          eff.stop = R.utils::doCall(eff.trial, args = c(list(eff.target = eff.target), eff.trial.control))
        } else {
          eff.stop = FALSE
        }
        if (!is.null(fut.arm)) {
          fut.stop = R.utils::doCall(fut.trial, args = c(list(fut.target = fut.target), fut.trial.control))
        } else {
          fut.stop = FALSE
        }
        #---
        # efficacy       
        if(any(mx.efficacy.lt[lw,aw])){
            # identify arms
            ew = which(mx.efficacy.lt[lw,]&aw)
            # inactive arms according to eff.trial
            id.target$active[ew] = FALSE
            id.group[id.target$group[ew],"active"] = FALSE
            # save estimate and adapt list of target 
            id.target$look[ew]     = lw
            id.target$efficacy[ew] = TRUE                
            id.target[ew,c("low","mid","high")] = fit$summary.fixed[id.target$id[ew],
                                                  c("0.025quant","mean","0.975quant")]
        }
        # futility
        if(any(mx.futility.lt[lw,aw])){
            # identify arms
            fw = which(mx.futility.lt[lw,]&aw)
            # inactive arms according to fut.trial
            id.target$active[fw] = FALSE
            id.group[id.target$group[fw],"active"] = FALSE
            # save estimate and adapt list of target 
            id.target$look[fw]     = lw
            id.target$futility[fw] = TRUE                
            id.target[fw,c("low","mid","high")] = fit$summary.fixed[id.target$id[fw],
                                                  c("0.025quant","mean","0.975quant")]
        }        
        # stop trial due to no active parameters or last look
        all.stop = (eff.stop|fut.stop)|
                   all(!id.target$active)| 
                   lw==n.look        
        if(all.stop){
            if(any(id.target$active)){
                aw = which(id.target$active)
                id.target$look[aw]     = lw
                id.target[aw,c("low","mid","high")] = fit$summary.fixed[id.target$id[aw],
                                                      c("0.025quant","mean","0.975quant")]
            }
            break
        # continue
        }else{
          
            # prob per group
            if(!is.null(RAR)){
                # prob per group
                assign("posterior",mx.posterior_RAR.lt[lw,id.target$active], envir = env)
                assign("active",id.group$active, envir = env)
                #prob = .eval(RAR,envir=env) 
                assign("n",unlist(id.look[lw, id.group$id]),envir = env)  
                #assign ingredients to environment 'env' 
                assign("ref",id.group$reference,envir = env)
                assign("N",id.look$n[n.look],envir = env)
                assign("RAR.control", RAR.control, envir = env)
                prob = R.utils::doCall(RAR, args = c(plyr::.(posterior=posterior,n=n,N=N,ref=ref,active=active), RAR.control) ,envir = env)      #call function RAR in environment 'env'
                #---
            }else{
                prob = prob0[id.group$active]
            }
            names(prob) = id.group$id[id.group$active]
            mx.rprob.lt[lw,names(prob)] = prob/sum(prob)
            
            # predictors
            assign("n",id.look[lw+1,"n"],envir=env)
            assign("m",id.look[lw+1,"m"],envir=env)
            assign("prob",prob,envir=env)
            set.seed(lw+(n.look+1)*int)
 
            assign("var.control",var.control,envir=env)
            covar <- vector("list",length(var[-1]))
            for (ii in 1:length(var[-1])) {
              tmp_nam <- names(var)[ii+1]
              args_ <- plyr::.(n=m,m=m,prob=prob)
              if (tmp_nam %in% names(var.control)) args_ <- c(args_, var.control[[tmp_nam]])
              covar[[ii]] <- R.utils::doCall(var[[ii+1]], envir = env, args = args_)    
              if (is.matrix(covar[[ii]])) colnames(covar[[ii]]) <- paste0(tmp_nam,1:dim(covar[[ii]])[2])
            }
            
            #---
            new = as.data.frame(matrix(NA,id.look[lw+1,"m"],n.var,
                                dimnames=list(paste0(lw+1,"-",1:id.look[lw+1,"m"]),id.var)))
            
            pos.col <- 2
            for (var.count in 1:(length(var)-1)){
              if (!is.matrix(covar[[var.count]])) {
                new[,pos.col] = covar[[var.count]]
                pos.col <- pos.col+1
              } else {
                for (jj in 1:dim(covar[[var.count]])[2]) {
                  new[,pos.col] = covar[[var.count]][,jj]
                  pos.col <- pos.col+1
                }
              }
            }
                
            # response
            X = model.matrix(model[-2], data = new)                                           # call model matrix from formula object 
            #---
            XB = X%*%beta[colnames(X)]
            assign("mu",switch(link,
                               "identity" = XB,
                               "log"      = exp(XB),
                               "logit"    = INLA::inla.link.logit(XB, inverse=TRUE),
                               "probit"   = INLA::inla.link.probit(XB, inverse=TRUE),
                               "robit"    = INLA::inla.link.robit(XB, inverse=TRUE),
                               "cauchit"  = INLA::inla.link.cauchit(XB, inverse=TRUE),
                               "loglog"   = INLA::inla.link.loglog(XB, inverse=TRUE),
                               "cloglog"  = INLA::inla.link.cloglog(XB, inverse=TRUE)),envir=env)
            
            tmp_nam <- names(var)[1] 
            args_ <- plyr::.(n=m,mu=mu)                                                             # create a quoted(!) list of available 'ingredients'
            if (!(identical(var[[1]],rbinom) || identical(var[[1]],rnbinom))) {
              names(args_)[1:2] <- formalArgs(var[[1]])[1:2]
            } else {
              if (identical(var[[1]],rbinom)) {
                names(args_)[1:2] <- c("n","prob")
              } else if (identical(var[[1]],rnbinom)) {
                names(args_)[1:2] <- c("n","mu")
              }
            }
            
            # rename the list objects to the names required by specified formula (NOTE: the order of the items is set, if a function requires a different order this will not work, clever rearranging may be needed)
            if (tmp_nam %in% names(var.control)) args_ <- c(args_, var.control[[tmp_nam]])          # add extra arguments if provided
            new[, id.var[1]] = R.utils::doCall(var[[1]], args = args_, envir = env)
            
            # data
            data = rbind(data,new)
       }# end continue
       #cat(".")
    }# end loop
    
    # output
    # cat(paste0("\t end:",int,"\n"))

    colnames(mx.rprob.lt)      = paste0("r(",colnames(mx.rprob.lt),")")
    colnames(id.look)[-c(1:4)] = paste0("n(",colnames(id.look)[-c(1:4)],")")
    colnames(mx.posterior_eff.lt)  = paste0("pe(",colnames(mx.posterior_eff.lt),")")
    colnames(mx.posterior_fut.lt)  = paste0("pf(",colnames(mx.posterior_fut.lt),")")
    list(target = id.target, look = cbind(id.look,mx.posterior_eff.lt,mx.posterior_fut.lt,mx.rprob.lt),
         data   = if(extended==2){data}else{NULL})
}

#' BATSS
#'
#' @name BATSS
#' @import plyr
#' @import R.utils
#' @import parallel
#' @import methods
#' @import stats
#' @import graphics
#' @import grDevices
#' @import abind
#' @importFrom graphics axis legend lines matplot matpoints mtext points abline layout par text title
#' @importFrom stats dnorm pnorm qnorm rnorm uniroot qbinom quantile var
#' @importFrom grDevices dev.new devAskNewPage gray
#' @importFrom methods is
#' @importFrom rlang dots_list
#' @importFrom plyr is.formula
#' @importFrom cli cli_h1 cli_h2 cli_h3
#' @importFrom utils packageVersion
NULL

#' @name batss.glm
#' @title Bayesian adaptive trial simulations for generalised linear models
#' @description Simulation of Bayesian adaptive trials with GLM endpoint using Integrated Nested Laplace Approximation (INLA).  
#' @param model an object of class '\link[stats]{formula}' indicating a symbolic description of the model to be fitted (as in the \link[stats]{lm} and \link[stats]{glm} functions).
#' @param var A list. Each entry corresponds to a variable described under '`model`' and indicates the name of a function allowing to generate variates (like \link[stats]{rnorm} and \link[stats]{rexp}, for example). The list names have to match the variable names unded in '`model`' and its first element should correspond to the model outcome. The grouping variable corresponding to the target parameters has to be of class '\link[base]{factor}' with levels corresponding to the names indicated in argument `prob0` (see below).
#' @param var.control An optional list of control parameters for the functions indicated in '`var`'. The names of the list items need to correspond to the names used in '`var`'. Each element is another list with names of the elements corresponding to the parameter names of the functions specified in '`var`'. 
#' @param family A character string indicating the name of the conditional distribution as described in the package INLA (check \link[INLA]{inla.list.models}). Default set to '`gaussian`'.
#' @param link A character string describing the link function to be used in the model to relate the outcome to the set of predictors: 'identity', 'log', 'logit', 'probit', 'robit', 'cauchit', 'loglog' and 'cloglog' are the currently available options. Default set to 'identity'.
#' @param beta A numerical vector of parameter values for the linear predictor. Its length has to match the number of column of the **X** matrix induced by the formula indicated under '`model`' (check \link[stats]{model.matrix}).
#' @param which A numerical vector indicating the position of the target `beta` parameters.
#' @param R a vector of natural numbers to be used as seeds (check \link[base]{set.seed}) for the different Monte Carlo trials (the vector length will thus correspond to the number of Monte Carlo trials). When `R` is a scalar, seeds `1` to `R` are used, where `R` corresponds to the number of Monte Carlo trials. 
#' @param alternative A vector of strings providing the one-sided direction of the alternative hypothesis corresponding to each target parameter indicated under '`which`' (in the same order). Possibilities are 'greater' (default) or 'less'. If the vector is of length 1, the same direction will be used for all target parameter tests.
#' @param RAR A function defining the response-adaptive randomisation probabilities of each group - reference group included - with the same group names and ordering as used in '`prob0`'. Arguments of this function will typically consider 'BATSS' ingredients. Check [RAR.trippa] and [RAR.optimal] for examples. If `RAR = NULL` (default), the probabilities/ratios indicated under `prob0` will be used throughout (fixed allocation probabilities).
#' @param RAR.control An optional list of control parameters for the function provided in '`RAR`'. 
#' @param N A scalar indicating the maximum sample size.
#' @param interim A list of parameters related to interim analyses. Currently, only '`recruited`' is available.  It consists in a vector of integers indicating the number of completed observations at each look, last excluded, in increasing order. Setting `interim = NA` specifies a fixed (non-adaptive) design with a single look at the maximum sample size `N`.
#' @param prob0 A named vector with initial allocation probabilities. Names need to correspond to the levels of the grouping variable. If `RAR = NULL`, these probabilities/ratios will be used throughout (fixed allocation probabilities).
#' @param delta.eff A vector (of length equal to the number of looks (i.e., number of interims + 1)) of clinically meaningful treatment effect values (on the linear predictor scale) to be used to define the efficacy-related posterior probabilities for each target parameter at each look. If a scalar is provided, the same value is used at each look. The default is `delta.eff = 0`. 
#' @param delta.fut A vector (of length equal to the number of looks (i.e., number of interims + 1)) of clinically meaningful treatment effect values (on the linear predictor scale) to be used to define the futility-related posterior probabilities for each target parameter at each look. If a scalar is provided, the same value is used at each look. The default is `delta.fut = delta.eff`. 
#' @param delta.RAR A vector (of length equal to the number of looks (i.e., number of interims + 1)) of clinically meaningful treatment effect values (on the linear predictor scale) to be used to define the RAR-related posterior probabilities for each target parameter at each look. If a scalar is provided, the same value is used at each interim analysis. The default is `delta.RAR = 0`. Note that, when a vector is provided, its last value is ignored as no randomisation is made at the last look.
#' @param eff.arm A function defining if efficacy has been achieved at a given look given the information available at that stage a given target parameter. The output of this function must be a \link[base]{logical} (of length 1). Arguments of this function will typically consider 'BATSS' ingredients. Check [eff.arm.simple] and [eff.arm.infofract] for examples. 
#' @param eff.arm.control An optional list of parameters for the function indicated in '`eff.arm`'.
#' @param eff.trial A function defining if the trial can be stopped for efficacy given the output of the function indicated in '`eff.arm`'. The output of this function must be a \link[base]{logical} of length one. Arguments of this function will typically only consider the 'BATSS' ingredient `eff.target`. Check [eff.trial.all] and [eff.trial.any] for examples. When `eff.trial = NULL` (default), the trial stops for efficacy when *all* target parameters are found to be effective (like in [eff.trial.all]). 
#' @param eff.trial.control An optional list of parameters for the function indicated in '`eff.trial`'.
#' @param fut.arm A function defining if futility has been achieved at a given look given the information available at that stage for each target parameter. The output of this function must be a \link[base]{logical} (of length 1). Arguments of this function will typically consider 'BATSS' ingredients. Check [fut.arm.simple] to see an example of such a function. 
#' @param fut.arm.control An optional list of parameters for the function indicated in '`fut.arm`'.
#' @param fut.trial A function defining if the trial can be stopped for futility given the output of the function indicated in '`fut.arm`'. The output of this function must be a \link[base]{logical} of length one. Arguments of this function will typically only consider the 'BATSS' ingredient `fut.target`. Check [fut.trial.all] for an example of such a function. When `fut.trial = NULL` (default), the trial stops for futility when *all* target parameters are found to be futile (like in [fut.trial.all]).
#' @param fut.trial.control An optional list of parameters for the function indicated in '`fut.trial`'.
#' @param H0 A logical indicating whether the simulation should also consider the case with all target parameters set to 0 to check the probability of rejecting the hypothesis that the target parameter value is equal to 0 individually (pairwise type I error) or globally (family-wise error rate). Default set to `H0=TRUE`.
#' @param computation A character string indicating how the computation should be performed. Possibilities are 'parallel' or 'sequential' with default `computation="parallel"` meaning that the computation is split between `mc.cores`. 
#' @param mc.cores An integer indicating the number of CPUs to be used when `computation="parallel"` (Default to 3 if no global '`mc.cores`' global option is available via \link[base]{getOption}).
#' @param extended an integer indicating the type of results to be returned. 0 (default) provides summary statistics, 1 adds the results of each Monte Carlo trial and 2 additionally returns each Monte Carlo dataset. [batss.combine] requires extended > 0 as the function needs to merge results of different sets of seeds.
#' @param ... Additional arguments to control fitting in \link[INLA]{inla}.
#' @returns The function [batss.glm] returns an S3 object of class 'batss' with available print/summary/plot functions
#' \itemize{
#'   \item beta - A data frame providing information related to the beta parameter vector, like parameter names and values, for example.
#'   \item look - A data frame providing information related to looks, like sample size of a given interim (m) and cumulative sample size at a given interim (n), for example.
#'   \item par - A list providing different information, like the used seeds (seed) and the groups (group), for example.
#'   \item H1 - A list providing trial results under the alternative, like the estimates per target parameter when the corresponding arm was stopped (estimate), the efficacy and futility probabilites per target parameter and overall (target, efficacy and futility), the sample size per group and trial (sample), the probabilities associated to each combination of efficacy and futility per group (scenario), the detailed results per trial (trial), for example.
#'   \item H0 - A list providing trial results under the global null hypothesis (same structure as H1).
#'   \item call - The matched call.
#'   \item type - The type of 'BATSS' analysis (only 'glm' is currently available).
#' }
#' @export
#' @seealso [summary.batss] and [plot.batss] for detailed summaries and plots, and [batss.combine] to combine different evaluations of [batss.glm] considering the same trial design but different sets of seeds (useful for cluster computation). 
#' @examples
#'\donttest{
#' # Example: 
#' # * Gaussian conditional distribution with sigma = 5
#' # * 3 groups with group means 'C' = 1 (ref), 'T1' = 2, 'T2' = 3,
#' #     where higher means correspond to better outcomes 
#' # * 5 interim analyses occurring when n = 100, 120, 140, 160, and 180
#' # * fixed and equal allocation probabilities per arm (i.e., no RAR)
#' # * max sample size = 200 
#' # * efficacy stop per arm when the prob of the corresponding parameter 
#' #     being greater than 0 is greater than 0.975 (?eff.arm.simple)
#' # * futility stop per arm when the prob of the corresponding parameter 
#' #     being greater than 0 is smaller than 0.05 (?fut.arm.simple) 
#' # * trial stop once all arms have stopped (?eff.trial.all and ?fut.trial.all)
#' #     or the max sample size was reached 
#'
#' sim = batss.glm(model            = y ~ group,   
#'                 var              = list(y     = rnorm,
#'                                         group = alloc.balanced),
#'                 var.control      = list(y = list(sd = 5)),
#'                 beta             = c(1, 1, 2),
#'                 which            = c(2:3),
#'                 alternative      = "greater",
#'                 R                = 20,
#'                 N                = 200,
#'                 interim          = list(recruited = seq(100, 180, 20)),
#'                 prob0            = c(C = 1/3, T1 = 1/3, T2 = 1/3),
#'                 eff.arm          = eff.arm.simple,
#'                 eff.arm.control  = list(b = 0.975),
#'                 fut.arm          = fut.arm.simple,
#'                 fut.arm.control  = list(b = 0.05),
#'                 computation      = "parallel",
#'                 H0               = TRUE,
#'                 mc.cores         = 2)# better: parallel::detectCores()-1
#' }
#' @export
batss.glm = function(
model,var,var.control=NULL,family="gaussian",link="identity",
beta,which,alternative = "greater",R=1e+4,N,interim,prob0,
delta.eff=0,delta.fut=delta.eff, delta.RAR=0,
eff.arm,eff.arm.control=NULL,
eff.trial=NULL,eff.trial.control=NULL,
fut.arm,fut.arm.control=NULL,
fut.trial=NULL,fut.trial.control=NULL,
RAR=NULL,RAR.control=NULL,
H0=TRUE,computation="parallel",
mc.cores=getOption("mc.cores", 3L),
#linux.os = NA,
extended=0, ...){

#---    
call <- match.call()                       # save call
model <- as.formula(model)                 # allow for string and formula input
#--- 
                
##
## dataset structure and useful definitions 
##

message("    Initialisation")    

#some checks
n = m = prob = NULL
#error messages
if (!is.character(model) && ! plyr::is.formula(model)) 
  stop("invalid 'model' argument")
if (!is.list(var) || !all(sapply(var,is.function)))
  stop("'var' must be a list of functions")
if (is.null(intersect(names(var),intersect(setdiff(unlist(strsplit(all.vars(model),"1")),all.vars(model)),
                                           setdiff(unlist(strsplit(all.vars(model),"2")),all.vars(model))))) && 
    !setequal(all.vars(model),names(var)))
  stop("all variables in the model formula must have a generating function in the 'var' list")
if (!(family %in% names(INLA::inla.models()$likelihood))){
  stop("invalid 'family' argument, see help files and inla documentation for available families")
} else {
  if (!(family %in% c("gaussian","binomial","nbinomial","poisson")))
    warning("functionality only tested for gaussian, binomial, negative binomial and poisson distributions")
}
if (!(link %in% INLA::inla.models()$likelihood[[family]]$link) || !(link %in% c("identity","log","logit","probit","robit","cauchit","loglog","cloglog")))
  stop("'link' not supported, see help files and inla documentation for available link functions")
if (identical(interim, NA)){
  interim.recruited = NULL
} else if (!is.null(interim)){
  if(!inherits(interim,"list")){stop("'interim' should be a list")}
  interim.recruited = interim$recruited
} else {
  stop("'interim' should be provided")
}
if (!is.null(RAR) && is.null(interim.recruited)){
  warning("'RAR' is not NULL but 'interim = NA' (fixed design): RAR will have no effect")
}
if (!is.null(interim.recruited) && !is.numeric(unlist(interim.recruited))) 
  stop("'interim.recruited' must be a (list of) numeric vector(s)")
if (!is.null(interim.recruited) && any(interim.recruited < 0)) 
  stop("negative interim recruitment numbers not allowed")
if ((N < 0) || length(N)>1) {
  stop("total sample size 'N' must be a positive scalar")
}
N <- floor(N)
if (length(which)>length(beta))
  stop("number of targets greater than number of parameters")
if ((!is.null(RAR) && !(is.function(RAR))) || 
    (!is.null(eff.arm) && !(is.function(eff.arm))) || (!is.null(eff.trial) && !(is.function(eff.trial))) ||
    (!is.null(fut.arm) && !(is.function(fut.arm))) || (!is.null(fut.trial) && !(is.function(fut.trial))))
  stop("'RAR', 'eff.arm', 'eff.trial', 'fut.arm', 'fut.trial' must be functions or NULL")

#warnings
if (!is.null(interim.recruited) && any(interim.recruited > N)) {
  warning("some interim analyses are outside the maximum sample size and will be ignored")
  interim.recruited <- interim.recruited[interim.recruited<N]
}
if (all(prob0 >= 0) && all(prob0 <= 1) && sum(prob0) != 1)
  warning("sum of 'prob0' not equal to 1")



# size per look
if(sum(!is.na(match(c("m0","m"),names(interim.recruited))))==2){
   size_look = seq(interim.recruited$m0,N,interim.recruited$m)  
   size_look[length(size_look)] = N
}else{
  interim.recruited <- sort(interim.recruited)        #sort interim recruited to make sure the values are ordered
   size_look = c(interim.recruited,N)  
   }
n.look  = length(size_look)   
id.look = data.frame(pos = 1:n.look,
                     id  = paste0("n=",size_look),
                     n   = size_look,
                     m   = c(size_look[1],size_look[-1]-size_look[-n.look]))
# generate predictors
env0 = new.env()
assign("m", id.look$m[1], envir = env0)
assign("n", id.look$n[1], envir = env0)
assign("prob",prob0,envir = env0)
    
assign("var.control", var.control, envir = env0)  
covar <- vector("list",length(var[-1]))
for (ii in 1:length(var[-1])) {
  tmp_nam <- names(var)[ii+1]
  args_ <- plyr::.(n=n,m=m,prob=prob)
  if (tmp_nam %in% names(var.control)) args_ <- c(args_, var.control[[tmp_nam]])
  covar[[ii]] <- R.utils::doCall(var[[ii+1]], envir = env0, args = args_)   #call functions directly
  if (is.matrix(covar[[ii]])) colnames(covar[[ii]]) <- paste0(tmp_nam,1:dim(covar[[ii]])[2])
}

n.var <- length(all.vars(model))
id.var <- names(var)
if (any(sapply(covar,is.matrix))) {
  where.mat <- which(sapply(covar,is.matrix))
  tmp_var <- covar[where.mat]
  id.var <- names(var)[1]
  for (ii in 1:length(covar)) {
    if (ii %in% (where.mat)) {
      id.var <- c(id.var,colnames(covar[[ii]]))
    } else {
      id.var <- c(id.var,names(var[ii+1]))
    }
  }
}

m0     = length(covar[[1]])
if(length(m0)>1|m0[1]!=id.look$m[1]){stop("different predictor length")}

data <- as.data.frame(matrix(NA,m0,n.var,
                     dimnames=list(paste0("1-",1:m0),id.var)))
pos.col <- 2
for (var.count in 1:(length(var)-1)){
    if (!is.matrix(covar[[var.count]])) {
      data[,pos.col] = covar[[var.count]]
      pos.col <- pos.col+1
    } else {
      for (jj in 1:dim(covar[[var.count]])[2]) {
        data[,pos.col] = covar[[var.count]][,jj]
        pos.col <- pos.col+1
      }
    }
}
# group
groupvar <- names(var)[2]
n.group  = nlevels(data[,groupvar])
id.group = data.frame(pos = 1:n.group,
                      id = levels(data[,groupvar]),
                      reference = levels(data[,groupvar])==levels(data[,groupvar])[1],
                      active = TRUE,
                      row.names = levels(data[,groupvar]))
# define covariate name corresponding to group
tmp = labels(terms(model))
whichw = rep(FALSE,length(tmp))
for(pw in 1:length(tmp)){
    if(is.factor(data[,tmp[pw]])){
        whichw[pw] = all(!is.na(match(id.group$id,levels(data[,tmp[pw]]))))&
                     all(!is.na(match(levels(data[,tmp[pw]]),id.group$id)))
        }
    }
if(sum(whichw)==1){
    groupvar = tmp[whichw]
}else{
    if(sum(whichw)==0){
        stop("the variable corresponding to the treatment isn't the expected factor")
    }else{
        stop("2 factors share the same levels")
    }
}
# look
id.look = cbind(id.look, 
                matrix(NA,n.look,n.group,
                dimnames=list(id.look$id,id.group$id)))
# generate X matrix for names
X = model.matrix(model[-2], data = data)                               #create model matrix straight from formula

if(ncol(X)!=length(beta)){stop("length of 'beta' not compatible with X matrix")}
names(beta) = colnames(X)
# targets (only defined once)
n.target  = length(which)
if(length(alternative)==1){alternative=rep(alternative,n.target)
}else{if(length(alternative)!=n.target){stop("length(alternative)!=n.target")}}
id.target = data.frame(pos = NA,
                       id  = colnames(X)[which],
                       alternative = alternative,
                       group = NA, active = TRUE, 
                       look = NA, efficacy = NA, futility = NA,
                       low = NA, mid = NA, high = NA,
                       row.names =  colnames(X)[which])
id.target$group = sapply(id.target$id,function(x){
    levels(data[,groupvar])[which(sapply(split(X[,x]!=0,data[,groupvar]),any))]
    })
id.target = id.target[order(id.target$group),]
id.target$pos = 1:n.target


# delta vector(s)
# mw = c(match("delta",names(eff.arm.control)),
#        match("delta",names(fut.arm.control)))

# none
if(identical(delta.fut,delta.eff)){
  twodelta = FALSE 
  if (length(delta.eff)==1) delta.eff = delta.fut = rep(delta.eff,n.look)
  if (length(delta.eff)!=n.look) stop("length of delta not equal to number of looks")
}else{
# both
if(!(is.null(eff.arm) || is.null(fut.arm))){  
    twodelta = TRUE        
    # efficacy
    if(length(delta.eff)==1){
      delta.eff = rep(delta.eff,n.look)
    }else{if(length(delta.eff)!=n.look){
        stop("length of delta not equal to number of looks")
    }}
    # futility
    if(length(delta.fut)==1){
      delta.fut = rep(delta.fut,n.look)
    }else{if(length(delta.fut)!=n.look){
        stop("length of delta not equal to number of looks")
    }}
# one
}else{
    twodelta = FALSE
    # unique
    #tmp = par[[mw[!is.na(mw)]]]
    tmp <- if (is.null(fut.arm)) delta.eff else delta.fut
    if(length(tmp)==1){
        tmp = rep(tmp,n.look)
    }else{if(length(tmp)!=n.look){
        stop("length of delta not equal to number of looks")
    }}
    # assign
    delta.eff = delta.fut = tmp
}}  

if (length(delta.RAR)==1) delta.RAR = rep(delta.RAR,n.look)
if (length(delta.RAR)!=n.look) stop("length of delta.RAR not equal to number of looks")

# trial stopping rules
if(is.null(eff.trial) && !is.null(eff.arm)){
    eff.trial = function(eff.target){all(eff.target)}             #use function directly
    #---
    }
if(is.null(fut.trial) && !is.null(fut.arm)){
    fut.trial = function(fut.target){all(fut.target)}             #use function directly
    #---
    }

# seeds
if(length(R)==1){id.seed=1:R}else{id.seed=R}    

#############################
# H1
#############################

if(!all(beta[which]==0)){

    message("    Evaluation of H1")           
    H1 = TRUE
    # sequential
    if(computation!="parallel"){
        trial_r = lapply(id.seed,batss.trial,
               data=data,model=model,link=link,family=family,beta=beta,
               RAR=RAR,RAR.control=RAR.control,twodelta=twodelta,delta.eff=delta.eff,delta.fut=delta.fut,
               eff.arm=eff.arm,eff.trial=eff.trial,delta.RAR=delta.RAR,
               eff.arm.control=eff.arm.control,eff.trial.control=eff.trial.control,
               fut.arm=fut.arm,fut.trial=fut.trial,
               fut.arm.control=fut.arm.control,fut.trial.control=fut.trial.control,
               id.target=id.target,n.target=n.target,
               id.look=id.look,n.look=n.look,prob0=prob0,
               id.group=id.group,n.group=n.group,groupvar=groupvar,
               var=var,var.control=var.control,id.var=id.var,n.var=n.var,
               #linux.os=linux.os,
               extended = extended, ...)
    # parallel
    }else{if(computation=="parallel"){           
        # unix via forking
        if(Sys.info()[[1]]!="Windows"){
        trial_r = parallel::mclapply(id.seed,batss.trial,
                                      data=data,model=model,link=link,family=family,beta=beta,
                                      RAR=RAR,RAR.control=RAR.control,twodelta=twodelta,delta.eff=delta.eff,delta.fut=delta.fut,
                                      eff.arm=eff.arm,eff.trial=eff.trial,delta.RAR=delta.RAR,
                                      eff.arm.control=eff.arm.control,eff.trial.control=eff.trial.control,
                                      fut.arm=fut.arm,fut.trial=fut.trial,
                                      fut.arm.control=fut.arm.control,fut.trial.control=fut.trial.control,
                                      id.target=id.target,n.target=n.target,
                                      id.look=id.look,n.look=n.look,prob0=prob0,
                                      id.group=id.group,n.group=n.group,groupvar=groupvar,
                                      var=var,var.control=var.control,id.var=id.var,n.var=n.var,
                                      # linux.os=linux.os,
                                      extended = extended, mc.cores=mc.cores,mc.set.seed = FALSE,...)
        # windows without forking
        }else{
        cl = parallel::makeCluster(getOption("cl.cores", mc.cores))
        parallel::clusterEvalQ(cl, c(library(INLA)))
        #parallel::clusterExport(cl, transfer, envir = .GlobalEnv)           
#        parallel::clusterExport(cl, c(".expit"), envir = environment())           
        trial_r = parallel::parLapply(cl=cl,id.seed,batss.trial,
                                      data=data,model=model,link=link,family=family,beta=beta,
                                      RAR=RAR,RAR.control=RAR.control,twodelta=twodelta,delta.eff=delta.eff,delta.fut=delta.fut,
                                      eff.arm=eff.arm,eff.trial=eff.trial,delta.RAR=delta.RAR,
                                      eff.arm.control=eff.arm.control,eff.trial.control=eff.trial.control,
                                      fut.arm=fut.arm,fut.trial=fut.trial,
                                      fut.arm.control=fut.arm.control,fut.trial.control=fut.trial.control,
                                      id.target=id.target,n.target=n.target,
                                      id.look=id.look,n.look=n.look,prob0=prob0,
                                      id.group=id.group,n.group=n.group,groupvar=groupvar,
                                      var=var,var.control=var.control,id.var=id.var,n.var=n.var,
                                      #linux.os=linux.os,
                                      extended = extended,...)
        stopCluster(cl)    
        }
    }}     
    
    
    ##
    ## results
    ##
    estimate = batss.res.e(trial_r,id.target)
    tar.p    = batss.res.tp(estimate,id.target)
    tar.g    = batss.res.tg(estimate,id.target)
    eff.p    = batss.res.ep(estimate,id.target,n.look)
    eff.g    = batss.res.eg(estimate,id.target,n.look)
    fut.p    = batss.res.fp(estimate,id.target,n.look)
    fut.g    = batss.res.fg(estimate,id.target,n.look)
    sample   = batss.res.s1(trial_r,group=id.group$id,
                           type=c(apply(estimate[,"type",,drop=FALSE],2:3,paste0,collapse="")),
                           early=c(apply(estimate[,"look",,drop=FALSE]<n.look,2:3,all)))
    scenario = batss.res.s2(sample,target=id.target$id)
    res_H1   = list(estimate = estimate,
                    target   = list(par=tar.p,global=tar.g),
                    efficacy = list(par=eff.p,global=eff.g),
                    futility = list(par=fut.p,global=fut.g),
                    sample=sample,scenario=scenario)
    trial_H1    = trial_r
}else{
    H1 = FALSE
}

#############################
# H0
#############################

if(H0==TRUE | all(beta[which]==0)){

    message("    Evaluation of H0")
    H0 = TRUE
    beta0 = beta
    beta0[which] = 0
    # sequential
    if(computation!="parallel"){
        trial_r = lapply(id.seed,batss.trial,
                         data=data,model=model,link=link,family=family,beta=beta0,
                         RAR=RAR,RAR.control=RAR.control,twodelta=twodelta,delta.eff=delta.eff,delta.fut=delta.fut,
                         eff.arm=eff.arm,eff.trial=eff.trial,delta.RAR=delta.RAR,
                         eff.arm.control=eff.arm.control,eff.trial.control=eff.trial.control,
                         fut.arm=fut.arm,fut.trial=fut.trial,
                         fut.arm.control=fut.arm.control,fut.trial.control=fut.trial.control,
                         id.target=id.target,n.target=n.target,
                         id.look=id.look,n.look=n.look,prob0=prob0,
                         id.group=id.group,n.group=n.group,groupvar=groupvar,
                         var=var,var.control=var.control,id.var=id.var,n.var=n.var,
                         #linux.os=linux.os,
                         extended = extended, ...)
    # parallel    
    }else{if(computation=="parallel"){           
        # unix via forking
        if(Sys.info()[[1]]!="Windows"){
        trial_r = parallel::mclapply(id.seed,batss.trial,
                                     data=data,model=model,link=link,family=family,beta=beta0,
                                     RAR=RAR,RAR.control=RAR.control,twodelta=twodelta,delta.eff=delta.eff,delta.fut=delta.fut,
                                     eff.arm=eff.arm,eff.trial=eff.trial,delta.RAR=delta.RAR,
                                     eff.arm.control=eff.arm.control,eff.trial.control=eff.trial.control,
                                     fut.arm=fut.arm,fut.trial=fut.trial,
                                     fut.arm.control=fut.arm.control,fut.trial.control=fut.trial.control,
                                     id.target=id.target,n.target=n.target,
                                     id.look=id.look,n.look=n.look,prob0=prob0,
                                     id.group=id.group,n.group=n.group,groupvar=groupvar,
                                     var=var,var.control=var.control,id.var=id.var,n.var=n.var,
                                     #linux.os=linux.os,
                                     extended = extended, 
                                     mc.cores=mc.cores,mc.set.seed = FALSE,...)
        # windows without forking
        }else{
        cl = parallel::makeCluster(getOption("cl.cores", mc.cores))
        parallel::clusterEvalQ(cl, c(library(INLA)))
        #parallel::clusterExport(cl, transfer, envir = .GlobalEnv)           
#        parallel::clusterExport(cl, c(".expit"), envir = environment())           
        trial_r = parallel::parLapply(cl=cl,id.seed,batss.trial,
                                      data=data,model=model,link=link,family=family,beta=beta0,
                                      RAR=RAR,RAR.control=RAR.control,twodelta=twodelta,delta.eff=delta.eff,delta.fut=delta.fut,
                                      eff.arm=eff.arm,eff.trial=eff.trial,delta.RAR=delta.RAR,
                                      eff.arm.control=eff.arm.control,eff.trial.control=eff.trial.control,
                                      fut.arm=fut.arm,fut.trial=fut.trial,
                                      fut.arm.control=fut.arm.control,fut.trial.control=fut.trial.control,
                                      id.target=id.target,n.target=n.target,
                                      id.look=id.look,n.look=n.look,prob0=prob0,
                                      id.group=id.group,n.group=n.group,groupvar=groupvar,
                                      var=var,var.control=var.control,id.var=id.var,n.var=n.var,
                                      #linux.os=linux.os,
                                      extended=extended, ...)
        stopCluster(cl)    
        }
    }}     
    
    ##
    ## results
    ##
    #browser()
    estimate = batss.res.e(trial_r,id.target)
    tar.p    = batss.res.tp(estimate,id.target)
    tar.g    = batss.res.tg(estimate,id.target)
    eff.p    = batss.res.ep(estimate,id.target,n.look)
    eff.g    = batss.res.eg(estimate,id.target,n.look)
    fut.p    = batss.res.fp(estimate,id.target,n.look)
    fut.g    = batss.res.fg(estimate,id.target,n.look)
    sample   = batss.res.s1(trial_r,group=id.group$id,
                           type=c(apply(estimate[,"type",,drop=FALSE],2:3,paste0,collapse="")),
                           early=c(apply(estimate[,"look",,drop=FALSE]<n.look,2:3,all)))
    scenario = batss.res.s2(sample,target=id.target$id)
    res_H0   = list(estimate = estimate,
                    target   = list(par=tar.p,global=tar.g),
                    efficacy = list(par=eff.p,global=eff.g),
                    futility = list(par=fut.p,global=fut.g),
                    sample=sample,scenario=scenario)
    trial_H0    = trial_r 
     
}else{
    H0 = FALSE
}

##
## output
##

message("    Results")    
look        = id.look[,c("pos","id","n","m")]
FE  = data.frame(pos=1:ncol(X),id=colnames(X),target=FALSE,
                 row.names = colnames(X))
FE[id.target$id,"target"] = TRUE
if(H0){FE[,'Beta (H0)'] = beta0}
if(H1){FE[,'Beta (H1)'] = beta}
# 
par = list(RAR=RAR, group=id.group[,c("pos","id","reference")],
           seed=id.seed, H0=H0, H1=H1, version=utils::packageVersion("BATSS"),
           eff.arm=eff.arm, eff.arm.control=eff.arm.control,
           eff.trial=eff.trial, eff.trial.control=eff.trial.control,
           fut.arm=fut.arm, fut.arm.control=fut.arm.control,
           fut.trial=fut.trial, fut.trial.control=fut.trial.control)
out = list(beta = FE, look = look, par=par)    
if(H0){
    out$H0 = res_H0
    if(extended>0){out$H0$trial = trial_H0}
    }
if(H1){
    out$H1 = res_H1
    if(extended>0){out$H1$trial = trial_H1}
    }
#---
out$call <- call
out$type <- "glm"
#---
class(out) = "batss"
out
}

#' @name plot.batss
#' @title Plot function for 'BATSS' outputs
#' @description Plot for objects of class 'batss' 
#' @param x An object of class 'batss' (i.e., output of the function [batss.glm]).
#' @param which An integer vector indicating the list of desired plots. If a subset of the plots is required, specify a subset of the numbers \code{1:4}. By default, all plots are provided, i.e., `which=1:4`. Plot `1` displays the boxplot of the total sample size as well as the boxplot of the sample sizes per group, Plot `2` displays a barplot of the probability of stopping at each look, plot `3` displays the violin plot of the sample size per group, and plot `4` displays the Monte Carlo trial target estimates as a function of the sample size.
#' @param ask A logical. If `TRUE`, the user is prompted to hit the \kbd{Enter} key before each plot. See \code{\link{par}(ask=.)}. The default is \code{ask=TRUE}.
#' @param hypothesis A character string indicating which alternative hypothesis to use for analyses considering both "H0" and "H1", with options "H1" (default) and "H0".
#' @param title Either a \link[base]{logical} indicating if a title should be added or a string (of class \link[base]{character}) indicating the title to be added. If `title` equals `TRUE` (default), the title 'Under 'H1' or 'Under 'H0' (depending on the argument `hypothesis`) is added to the outer margin of the plot. No outer margin space is added if `title = FALSE`.
#' @param legend A \link[base]{logical} (with default set to `TRUE`) if a legend should be added at the bottom of plots 3 and 4, or a list with names `height`, `cex` and `pt` respectively indicating i/ the fraction of the plot to be used for the legend as a numeric (default is `.15`), ii/ the character expansion factor relative to current `par("cex")` as a numeric (default to `1.25`), and iii/ the expansion factor(s) for the points as a numeric (default to `2`). The input `legend = TRUE` is equivalent to `legend = c(height=.15, cex=1, pt=2)`.
#' @param ess A \link[base]{logical} (with default set to `TRUE`) indicating if the expected sample size should be displayed in plots 2, 3 and 4, or a list with names `col`, `cex` and `bg` respectively indicating i/ the colour of the label as a character (plots 2, 3, and 4), ii/ the text expension level as a numerical value (plots 2, 3 and 4) and iii/ the background colour as a character (plot 3). The input `ess = TRUE` is equivalent to `ess = list(col="blue", cex=1, bg="#FFD70070")`.
#' @param percentage A \link[base]{logical} (with default set to `TRUE`) indicating if the probability of stopping at each look should be displayed in plots 2 (as a percentage), or a list with names `col`, and `cex` indicating i/ the colour of the label as a character, ii/ the text expension level as a numerical value. The input `percentage = TRUE` is equivalent to `percentage = list(col="violet", cex=1)`.
#' @param beta A \link[base]{logical} (with default set to `TRUE`) indicating if the assumed target parameter value(s) should be displayed in plot 4, or a list with names `col`, `lwd`, and `lty` respectively indicating i/ the colour of the target parameter value horizontal line(s) as a character, ii/ the thickness of the line(s) as a numerical value  and iii/ the type of line as an integer (see \link[graphics]{par} for details). The input `beta = TRUE` is equivalent to `beta = list(col="blue", lwd=1.5, lty=1)`.
#' @param col A vector of length 4 specifying the colours to be used. Default to `c("#8B897040","#008B0040","#8B3A3A40","#FF990075")` where the last two digits of the hexadecimal strings specify the level of transparency. Refer to the Section 'colour specification' in \link[graphics]{par} for details. If the length of `col` equals 1, the same colour is used for all cases. For plots `1` and `2`, the 3rd colour of the vector `col` is used to display the barplot and boxplots. 
#' @param smooth A numerical (>0) indictating the level of smoothing of the violin plots (for plot `3`). Default to `1`. When `smooth=NULL`, the smoothing value is optimised in the \link[sm]{sm.density} function.
#' @param ... Additional arguments affecting the plot produced, like ylim and ylab.
#' @returns Generates graphical displays of results for objects of class 'batss'.
#' @seealso [batss.glm()], the function generating S3 objects of class 'batss'. 
#' @export
plot.batss = function(x, which=1:4, ask=TRUE, hypothesis="H1", 
                      title=TRUE, legend=TRUE, ess=TRUE, percentage=TRUE, beta=TRUE, 
                      col = c("#008B0040","#8B3A3A40","#8B897040","#FF990075"),
                      smooth = 1, ...){
    which.plot=rep(TRUE,4)
    if(!is.null(which)){which.plot[-which]=FALSE}    
    if(sum(which.plot)==1){ask=FALSE}    
    if(ask){oask <- devAskNewPage(TRUE)
    on.exit(devAskNewPage(oask))
    }
    oldpar <- par(no.readonly = TRUE)
    on.exit(par(oldpar), add = TRUE)
    mc = match.call()    
    # hypothesis 
    if(is.na(match(hypothesis,names(x)))){
        stop("'",hypothesis,"' not available in 'batss' object")  
        }
    # title
    if(is.character(title)){
        titlew = title
        title  = TRUE
    }else{
        if(is.logical(title)){
            if(title){titlew = paste0("Under '",hypothesis,"'")}
        }else{
            warning("title should be a logical or a character string")
            title = FALSE
        }
    }     
    # col
    if(length(col)==1){col = rep(col,4)
    }else{
        if(length(col)<4){
            col[(length(col)+1):4] = c("#008B0040","#8B3A3A40","#8B897040","#FF990075")[(length(col)+1):4]
        }
    }
    # legend
    if(is.list(legend)){# legend = list(col="green", cex=1.25)
        legend.height = ifelse(!is.null(legend[["height"]]),legend[["height"]],.15)        
        legend.cex    = ifelse(!is.null(legend[["cex"]]),legend[["cex"]],1)    
        legend.pt     = ifelse(!is.null(legend[["pt"]]),legend[["pt"]],2)    
        legend        = TRUE
    }else{
        if(!is.logical(legend)){
            stop("legend should be a list or a logical")
        }else{
            if(legend){
                legend.height = .15      
                legend.cex    = 1
                legend.pt     = 2
            }
        }
    }
    # ess
    if(is.list(ess)){# ess = list(col="green", cex=1.25)
        ess.col = ifelse(!is.null(ess[["col"]]),ess[["col"]],"blue")        
        ess.cex = ifelse(!is.null(ess[["cex"]]),ess[["cex"]],1)    
        ess.bg  = ifelse(!is.null(ess[["bg"]]),ess[["bg"]],"#FFD70070")    
        ess     = TRUE
    }else{
        if(!is.logical(ess)){
            stop("ess should be a list or a logical")
        }else{
            if(ess){
                ess.col = "blue"       
                ess.cex = 1
                ess.bg  = "#FFD70070"
            }
        }
    }
    # percentage
    if(is.list(percentage)){# perc = list(col="green", cex=1.25)
        percentage.col = ifelse(!is.null(percentage[["col"]]),percentage[["col"]],"violet")        
        percentage.cex = ifelse(!is.null(percentage[["cex"]]),percentage[["cex"]],1)    
        percentage     = TRUE
    }else{
        if(!is.logical(percentage)){
            stop("perc should be a list or a logical")
        }else{
            if(percentage){
                percentage.col = "violet"       
                percentage.cex = 1
            }
        }
    }    
    # beta
    if(is.list(beta)){#
        beta.col = ifelse(!is.null(beta[["col"]]),beta[["col"]],"blue")
        beta.lwd = ifelse(!is.null(beta[["lwd"]]),beta[["lwd"]],1.5)
        beta.lty = ifelse(!is.null(beta[["lty"]]),beta[["lty"]],1)
        #beta     = TRUE
    }else{
        if(!is.logical(beta)){
            stop("beta should be a list or a logical")
        }else{
            if(beta){
                beta.col = "blue"       
                beta.lwd = 1.5
                beta.lty = 1
            }
        }
    }        
    # prep
    id.targetw = x[[hypothesis]]$target$par
    n.targetw  = nrow(id.targetw)
    id.targetw$beta = x$beta[id.targetw$id,grepl(hypothesis,colnames(x$beta))]
    ar.inf.rt4 = array(dim=c(dim(x[[hypothesis]]$sample)[1],n.targetw,4),
                       dimnames=list(1:dim(x[[hypothesis]]$sample)[1],
                                     x[[hypothesis]]$target$id,
                                     c("n","est","type","last")))
    ar.inf.rt4[,,"n"]    = as.matrix(x[[hypothesis]]$sample[,x[[hypothesis]]$target$par$group]) 
    ar.inf.rt4[,,"est"]  = t(x[[hypothesis]]$estimate[,"mid",])
    ar.inf.rt4[,,"type"] = t(x[[hypothesis]]$estimate[,"type",])
    ar.inf.rt4[,,"last"] = t(x[[hypothesis]]$estimate[,"look",]==nrow( x$look))
    #ar.inf.rt4 = ar.inf.rt4[order(x[[3]]$seed),,]
    mx.n.rg = x[[hypothesis]]$sample[,-(ncol(x[[hypothesis]]$sample)+(-1:0))]   
    n.group = ncol(mx.n.rg)     
    ##
    ## boxplots of all sample sizes
    ##
    if(which.plot[1]){
        cex  = if(!any(names(mc)=="cex")){1}else{eval(mc$cex)}        
        ylim = if(!any(names(mc)=="ylim")){range(x$look$n)}else{eval(mc$ylim)}
        ylab = if(!any(names(mc)=="ylab")){"Sample size"}else{mc$ylab}
        layout(matrix(1:2,ncol=2),widths=c(2,n.group),heights=1)
        par(mar=c(2,4,3,0.25),omi=c(0,0,ifelse(title,0.25,0),.05))        
        #
        boxplot(apply(mx.n.rg,1,sum),ylab=ylab,xlab="",main="Total",
                ylim=ylim,axes=FALSE,col=col[3],cex=cex)
        box()       
        axis(2,las=2,cex.axis=.75)   
        axis(2,at=max(x$look$n),las=2)       
        #
        boxplot(mx.n.rg,ylab=ylab,xlab="",main="Per group",
                axes=FALSE,col=col[3])
        box()       
        axis(2,las=2)       
        axis(1,at=1:n.group,colnames(mx.n.rg))       
        #
        if(title){mtext(titlew,3,outer=TRUE,font=2,cex=1.5)}             
    }
    ##
    ## barplot of total sample size
    ##
    if(which.plot[2]){
        # 
        total.r = apply(mx.n.rg,1,sum)        
        x$look$pi.hat = 0
        x$look$pi.hat[match(names(table(total.r)),x$look$n)] = table(total.r)/nrow(mx.n.rg)*100
        n.look = nrow(x$look)
        shift  = mean(x$look$n[-1]-x$look$n[-n.look])*0.2
        # options
        cex  = if(!any(names(mc)=="cex")){1}else{eval(mc$cex)}        
        ylim = if(!any(names(mc)=="ylim")){max(x$look$pi.hat)*c(0,1.05)}else{eval(mc$ylim)}
        xlim = if(!any(names(mc)=="xlim")){range(x$look$n)+c(-shift,shift)}else{eval(mc$xlim)}
        ylab = if(!any(names(mc)=="ylab")){"Frequency (%)"}else{mc$ylab}
        xlab = if(!any(names(mc)=="xlab")){"Cumulative total sample size per look (n)"}else{mc$xlab}        
        font = if(!any(names(mc)=="font")){2}else{eval(mc$font)}        
        lty  = if(!any(names(mc)=="lty")){2}else{eval(mc$lty)}                
        #
        layout(matrix(1:1,ncol=1))
        par(mar=c(4,4,1,0.25),omi=c(0,0,ifelse(title,0.25,0),.05))     
        plot(1, 1, pch = "", axes = FALSE, xlab = "", ylab = "",
             main = "", ylim = ylim, xlim = xlim)
        abline(h=0)
        #
        pos_ticks <- pretty(x$look$pi.hat, n=5)
        pos_ticks <- pos_ticks[pos_ticks >= 0 & pos_ticks< ylim[2]]
        axis(2,at=pos_ticks, las=2)        
        axis(2,at=ylim[2]/2,ylab,tick=FALSE,font=font,padj=-3)
        #
        axis(1,at=x$look$n, paste0(x$look$n,"\n(#",x$look$pos,")"), pos=0,padj=0.5)        
        axis(1,at=mean(xlim),xlab,tick=FALSE,font=font,padj=3.5,pos=0)        
        #
        for(lw in 1:n.look){
            rect(x$look$n[lw]-shift,0,
                 x$look$n[lw]+shift,x$look$pi.hat[lw],col=col[3])
        }
        if(percentage){
            for(lw in 1:n.look){
                text(x$look$n[lw],x$look$pi.hat[lw],
                     paste0(format(round(x$look$pi.hat[lw],2)),"%"),
                     col=percentage.col,pos=3,cex=percentage.cex)
            }
        }
        #
        if(ess){
            segments(mean(total.r),0,
                     mean(total.r),ylim[2],col=ess.col,lty=lty)   
            axis(3,at=mean(total.r),paste0("ESS = ",round(mean(total.r),1)),
                 col.axis=ess.col, tick=FALSE, pos=ylim[2], cex=ess.cex) 
        }
        # 
        if(title){mtext(titlew,3,outer=TRUE,font=2,cex=1.5)}                      
    }
    ##
    ## violin plots
    ##
    if(which.plot[3]){
        # options
        cex  = if(!any(names(mc)=="cex")){1}else{eval(mc$cex)}        
        ylim = if(!any(names(mc)=="ylim")){range(mx.n.rg)*c(0,1)}else{eval(mc$ylim)}
        xlim = if(!any(names(mc)=="xlim")){c(1,n.group)+c(-.5,.5)}else{eval(mc$xlim)}
        ylab = if(!any(names(mc)=="ylab")){"Sample size (n)"}else{mc$ylab}
        #xlab = if(!any(names(mc)=="xlab")){""}else{mc$xlab}        
        font = if(!any(names(mc)=="font")){2}else{eval(mc$font)}        
        lty  = if(!any(names(mc)=="lty")){2}else{eval(mc$lty)}                
        pch  = if(!any(names(mc)=="pch")){c(19,4)}else{eval(mc$pch)}        
        h    = if(!any(names(mc)=="h")){c(1)}else{eval(mc$h)}        
        pos_ticks <- pretty(c(ylim[1],as.matrix(mx.n.rg)), n=8)
        ylim = if(!any(names(mc)=="ylim")){range(pos_ticks)}
        lwd  = if(!any(names(mc)=="lwd")){1.25}

        if(legend){
            layout(matrix(1:2,ncol=1),heights=c(1,legend.height))
        }else{
            layout(matrix(1:1,ncol=1))
        }
        par(mar=c(1.5,4,0.25,0.25),omi=c(0,0,ifelse(title,0.25,0),.05))  
        plot(1, 1, pch = "", axes = FALSE, xlab = "", ylab = "",
             main = "", ylim = ylim, xlim = xlim)
        abline(h=pos_ticks,col=gray(.85),lwd=.85)
        axis(2,las=2,at=pos_ticks)
        axis(2,at=mean(ylim),ylab,tick=FALSE,font=font,padj=-3)        
        axis(1,at=1:n.group,colnames(mx.n.rg),pos=0)

        for(gw in 1:n.group){   # gw=1        
            if(any(id.targetw$group==colnames(mx.n.rg)[gw])){
                colw  = col[c(3,1,2,4)][ar.inf.rt4[,which(id.targetw$group==colnames(mx.n.rg)[gw]),"type"]+1]
            }else{
                colw  = col[3]
            }
            violin    = violin.density(mx.n.rg[,gw], h=smooth)
            # points
            epsilon.x = rnorm(nrow(mx.n.rg),rep(0,nrow(mx.n.rg)),
                              sqrt(violin$pw.freq/max(violin$pw.freq))*.15)
            epsilon.y = runif(nrow(mx.n.rg),-.25,.25)
            points(rep(gw,nrow(mx.n.rg))+epsilon.x,mx.n.rg[,gw]+epsilon.y,
                   col=colw,pch=pch)
            # violin
            xx = c(-violin$sm.density, violin$sm.density[length(violin$sm.density):1])+gw
            yy = c(violin$sm.points, violin$sm.points[length(violin$sm.points):1])
            polygon(xx, yy, col = NA, border = 1, lwd=1.75)
            # boxplot
            segments(gw,violin$quantile[1],gw,violin$quantile[2],lwd=1)
            segments(gw,violin$quantile[4],gw,violin$quantile[5],lwd=1)
            rect(gw-.025, violin$quantile[2], gw+.025, violin$quantile[4], 
                 col=paste0(gray(.99),99), border=1)
            segments(gw-.025,violin$quantile[3],
                     gw+.025,violin$quantile[3],col=1, lwd=2)
            # ess
            if(ess){
                points(gw,violin$mean,pch=15,col=ess.col)
                #segments(gw-.025,violin$mean,
                #         gw+.025,violin$mean,col=col[5], lwd=5)
                label        = paste0("ESS=",round(violin$mean,1))
                label_width  = strwidth(label,cex=ess.cex)
                label_height = strheight(label,cex=ess.cex)
                rect(gw+.05, violin$mean - label_height*3/4, 
                     gw+.05+label_width*11/10, violin$mean + label_height*3/4, 
                     col = ess.bg, border = NA)
                text(gw,violin$mean,pos=4,label,
                     col=ess.col, cex=ess.cex)
            }
        }
        #
        if(title){mtext(titlew,3,outer=TRUE,font=2,cex=1.5)}     
        if(legend){            
            par(mar=c(0,4,0,0))
            plot(1, 1, pch = "", axes = FALSE, xlab = "", ylab = "",
                 main = "", ylim = c(0,1), xlim = c(0,1))
            if(!any(ar.inf.rt4[,,"type"]==3)){
                if(all(nchar(col)[1:3]==9)){colw = substr(col[1:3],1,7)}else{colw=col}
                legend("top",legend=c("Efficacy","Futility","Neither"),
                       col=colw,pch=15,box.lwd=NA,ncol=3,pt.cex=legend.pt,cex=legend.cex)
                }else{
                if(all(nchar(col)[1:4]==9)){colw = substr(col[1:4],1,7)}else{colw=col}
                legend("top",legend=c("Efficacy","Futility","Neither","Both"),
                       col=colw,pch=15,box.lwd=NA,ncol=4,pt.cex=legend.pt,cex=legend.cex)                
            }
            legend("bottom",legend=c("Early stopping","Stopping at last look"),
                   col=1,pch=pch,box.lwd=NA,ncol=2,pt.cex=legend.pt,cex=legend.cex)
        }
    }
    # estimates
    if(which.plot[4]){
        cex   = if(!any(names(mc)=="cex")){1}else{eval(mc$cex)}
        pch   = if(!any(names(mc)=="pch")){c(19,4)}else{eval(mc$pch)}        
        ylimw = if(!any(names(mc)=="ylim")){range(ar.inf.rt4[,,"est"])}else{eval(mc$ylim)}
        xlimw = range(ar.inf.rt4[,,"n"])
        if(legend){
            layout(rbind(matrix(1:n.targetw,ncol=n.targetw),n.targetw+1),
                   heights=c(1,legend.height))
        }else{
            layout(matrix(1:n.targetw,ncol=n.targetw))
        }
        par(mar=c(4,4,3,1),omi=c(0,0,ifelse(title,0.25,0),.05))
        for(tw in 1:n.targetw){
            dataw = data.frame(n=ar.inf.rt4[,tw,"n"],
                               est=ar.inf.rt4[,tw,"est"],
                               type=ar.inf.rt4[,tw,"type"],
                               last=ar.inf.rt4[,tw,"last"],
                               col=col[c(3,1,2,4)][ar.inf.rt4[,tw,"type"]+1],
                               pch=pch[ar.inf.rt4[,tw,"last"]+1])
            plot(dataw$n,dataw$est,
                 col=dataw$col,ylim=ylimw,xlim=xlimw,
                 main=id.targetw$id[tw],xlab="Sample size",
                 ylab = "Estimates",pch=dataw$pch,cex=cex)
            abline(h= id.targetw$beta[tw],col=beta.col,lty=beta.lty, lwd=beta.lwd)  
            #if(!is.null(beta.label){
            #    text(xlimw[1]+(xlimw[2]-xlimw[1]*.05),id.targetw$beta[tw],
            #         beta.label, col=beta.col, cex=beta.cex, pos=3)
            #}
            if(ess){   
                abline(v= mean(ar.inf.rt4[,tw,"n"]),col=ess.col,lty=3)
                axis(3,mean(ar.inf.rt4[,tw,"n"]),tick=FALSE,
                     paste0("ESS = ",round(mean(ar.inf.rt4[,tw,"n"]),1)),
                     col.axis=ess.col,padj=2,cex.axis=ess.cex)
            }
        }
        if(title){mtext(titlew,3,outer=TRUE,font=2,cex=1.5)}     
        if(legend){            
            par(mar=c(0,4,0,0))
            plot(1, 1, pch = "", axes = FALSE, xlab = "", ylab = "",
                 main = "", ylim = c(0,1), xlim = c(0,1))
            if(!any(dataw$type==3)){
                if(all(nchar(col)[1:3]==9)){colw = substr(col[1:3],1,7)}else{colw=col}
                legend("top",legend=c("Efficacy","Futility","Neither"),
                       col=colw,pch=15,box.lwd=NA,ncol=3,pt.cex=legend.pt,cex=legend.cex)
                }else{
                if(all(nchar(col)[1:4]==9)){colw = substr(col[1:4],1,7)}else{colw=col}
                legend("top",legend=c("Efficacy","Futility","Neither","Both"),
                       col=colw,pch=15,box.lwd=NA,ncol=4,pt.cex=legend.pt,cex=legend.cex)                
            }
            legend("bottom",legend=c("Early stopping","Stopping at last look"),
                   col=1,pch=pch,box.lwd=NA,ncol=2,pt.cex=legend.pt,cex=legend.cex)
        }
    }
}


#' @name violin.density
#' @title Violin plot density estimates
#' @description Extracts density estimates for a violin plot. 
#' @param data data vector.
#' @param h a scalar corresponding to the smoothing parameter.
#' @returns a list of coordinates and corresponding density values.
#' @seealso \link[sm]{sm.density} 
violin.density = function(data, h=NULL){
    data      = data[!is.na(data)]
    quantilew = quantile(data, prob=c(0,.25,.5,.75,1))
    iqrw      = quantilew[4] - quantilew[2]
    upper     = min(quantilew[4] + 1.5 * iqrw, quantilew[5])
    lower     = max(quantilew[2] - 1.5 * iqrw, quantilew[1])
    rangew    = c(min(lower, quantilew[1]), max(upper, quantilew[5]))
    argw      = list(display="none",h=h)
    # sm
    if(is.null(h)){
        smoothw   = do.call(sm::sm.density, c(list(data, xlim = rangew, display="none")))
    }else{
        smoothw   = do.call(sm::sm.density, c(list(data, xlim = rangew, display="none", h=h)))
    }
    # block
    n.lim   = min(100,length(unique(data)))
    lim     = seq(quantilew[1],quantilew[5],length=n.lim)
    pw.data = rep(n.lim, length(data))
    for(lw in n.lim:1){
        pw.data[data<=lim[lw]] = lw
    }
    pw.freq = tabulate(pw.data)/length(data)
    # 
    list(sm.points  = smoothw$eval.points,
         sm.density = smoothw$estimate * 0.4/max(smoothw$estimate),
         quantile   = c(lower,quantilew[2:4],upper), range=rangew, mean=mean(data),
         data       = data, pw.freq = pw.freq[pw.data])
}

#' @name summary.batss
#' @title Summary function for 'BATSS' outputs
#' @description Summary method function for objects of class 'batss'.
#' @param object An object of class 'batss' (i.e., output of the function [batss.glm]).
#' @param extended A logical indicating if a standard (extended = FALSE, default) or extended output (extended = TRUE) should be returned. Default to `NULL` in which case the input of the argument `extended` chosen when generating `object` with [batss.glm()] is used.
#' @param ... Additional arguments affecting the summary produced.
#' @returns Object of class 'summary.batss'.
#' @returns The function [summary.batss] returns an S3 list of class 'summary.batss' with available print functions. The list elements are
#' \itemize{
#'   \item beta - A data frame providing information related to the beta parameter vector, such as parameter names and values, for example.
#'   \item look - A data frame providing information related to looks, like sample size of a given interim (m) and cumulative sample size at a given interim (n), for example.
#'   \item par - A list providing different information, like the used seeds (seed) and the groups (group), for example.
#'   \item H1 - A list providing trial aggregated results under the alternative, like the probability of efficacy, futility, or both, per arm or globally (`object$H1$target`), the probability of stopping early for efficacy (`object$H1$efficacy`) and futility (`object$H1$futility`), the sample size expectation, standard deviation, and quantiles 0.1, 0.5 and 0.9, per group and overall (`object$H1$summary.sample.sizes`), the probabilities associated to each combination of efficacy and futility per group (scenario).
#'   \item H0 - A list providing trial aggregated results under the global null hypothesis (same structure as H1).
#'   \item call - The matched call.
#'   \item type - The type of 'BATSS' analysis (only 'glm' is currently available).
#' }
#' @seealso [batss.glm()], the function generating S3 objects of class 'batss'. 
#' @export
summary.batss = function(object, extended=NULL, ...){

  res          <- list()
  res$call     <- object$call
  res$par      <- object$par
  res$type     <- object$type
  res$beta     <- object$beta
  res$sample   <- object$look
  colnames(res$sample)[1] = ""
  if(is.null(extended)){
    res$extended   <- ifelse(is.null(object$call$extended),FALSE,object$call$extended>0)
  }else{
    res$extended   <- as.numeric(extended) 
  }

  if (object$par$H0) {
    
    # sample sizes
    res$H0$sample.sizes <- object$H0$sample[,1:nrow(object$par$group)]
    # target parameters
    res$H0$target <- object$H0$target
    temp = rbind(object$H0$target$par,object$H0$target$global)
    if(all(temp$both==0)){temp = temp[,colnames(temp)!="both"]}       
    res$H0$target <- temp
    # efficacy    
    temp = rbind(object$H0$efficacy$par,object$H0$efficacy$global)
    temp[,1][-(1:nrow(object$H0$efficacy$par))] = ""
    res$H0$efficacy <- temp
    # futility
    temp = rbind(object$H0$futility$par,object$H0$futility$global)
    temp[,1][-(1:nrow(object$H0$futility$par))] = ""
    res$H0$futility <- temp
    
    if(res$extended>0){    
      # ss
      temp = object$H0$sample[,1:nrow(object$par$group)]
      temp = cbind(temp,Total=apply(temp,1,sum))
      temp = data.frame(pos=1:ncol(temp),
                        id=colnames(temp),
                        'ESS'=round(apply(temp,2,mean),2),
                        'St.Dev'=round(sqrt(apply(temp,2,var)),2),
                        'q10'=round(apply(temp,2,quantile,probs=0.1),2),
                        'q50'=round(apply(temp,2,quantile,probs=0.5),2),
                        'q90'=round(apply(temp,2,quantile,probs=0.9),2))
      colnames(temp)[c(1,5:7)] = c("",paste0("q(",c(0.1,0.5,0.9),")"))
      temp[,1][nrow(temp)] = ""
      res$H0$summary.sample.sizes <- temp
      # scenarios    
      res$H0$scenario <- object$H0$scenario
    }    
  }else{
    res$H0 <- NULL 
  } 
  
  if(object$par$H1){

    # sample sizes
    res$H1$sample.sizes <- object$H1$sample[,1:nrow(object$par$group)]
    # target parameters
    res$H1$target <- object$H1$target
    temp = rbind(object$H1$target$par,object$H1$target$global)
    if(all(temp$both==0)){temp = temp[,colnames(temp)!="both"]}       
    res$H1$target <- temp
    # efficacy    
    temp = rbind(object$H1$efficacy$par,object$H1$efficacy$global)
    temp[,1][-(1:nrow(object$H1$efficacy$par))] = ""
    res$H1$efficacy <- temp
    # futility
    temp = rbind(object$H1$futility$par,object$H1$futility$global)
    temp[,1][-(1:nrow(object$H1$futility$par))] = ""    
    res$H1$futility <- temp

    if(res$extended>0){    
      # ss
      temp = object$H1$sample[,1:nrow(object$par$group)]
      temp = cbind(temp,Total=apply(temp,1,sum))
      temp = data.frame(pos=1:ncol(temp),
                        id=colnames(temp),
                        'ESS'=round(apply(temp,2,mean),2),
                        'St.Dev'=round(sqrt(apply(temp,2,var)),2),
                        'q10'=round(apply(temp,2,quantile,probs=0.1),2),
                        'q50'=round(apply(temp,2,quantile,probs=0.5),2),
                        'q90'=round(apply(temp,2,quantile,probs=0.9),2))
      colnames(temp)[c(1,5:7)] = c("",paste0("q(",c(0.1,0.5,0.9),")"))
      temp[,1][nrow(temp)] = ""
      
      res$H1$summary.sample.sizes <- temp
      # scenarios    
      res$H1$scenario <- object$H1$scenario
    } 
  } else {
    res$H1 <- NULL 
  } 
  class(res) = "summary.batss"
  return(res)
}
  
#' @name print.summary.batss
#' @title Print function for objects of class 'summary.batss' 
#' @description Print function for objects of class 'summary.batss' 
#' @param x An object of class 'summary.batss' (i.e., output of the function [summary] used on an output of the function [batss.glm]).
#' @param ... Additional arguments affecting the summary produced.
#' @returns Prints a summary for objects of class 'batss'.
#' @seealso [batss.glm()], the function generating S3 objects of class 'batss'. 
#' @export 
print.summary.batss = function(x, ...){
  object = x
  cat("\n")
  if(nrow(object$sample) > 1){
    cli_h1("Bayesian Adaptive Design with Laplace Approx.")
  }else{
    cli_h1("Bayesian Fixed Design with Laplace Approx.")
  }
  cat("  (",length(object$par$seed)," Monte Carlo samples)\n",sep="")
  cat("\n")
  cli_h3("Variables:")
  for(i in 2:length(object$call$var)){
    cat("  *",names(object$call$var)[i],":",as.character(object$call$var)[i],"\n")
  }
  if(!is.null(object$par$RAR)){
    cat("\n")
    cli_h3("Group randomisation:")
    cat("  *",object$call$RAR,"\n")
  }
  if(!(is.null(object$par$eff.arm) && is.null(object$par$fut.arm))){
    cat("\n")
    cli_h3("Decision rules:")
    if(!is.null(object$par$eff.arm)) cat("  * Efficacy: ",format(object$call$eff.arm),"\n")
    if(!is.null(object$par$fut.arm)) cat("  * Futility: ",format(object$call$fut.arm),"\n")
  }
  cat("\n")
  cli_h3("Model: ")
  cat("  *",format(object$call$model),paste("(with",ifelse(is.null(object$call$link),"identity",object$call$link), "link)\n"))
  cat("\n")
  cli_h3("Fixed effect parameters:\n")
  objectw = object$beta
  colnames(objectw)[1] = ""
  print(objectw,row.names=FALSE)
  cat("\n")
  cli_h3("Sample size per interim analyis:\n")
  objectw = object$sample
  colnames(objectw)[1] = ""
  print(objectw,row.names=FALSE)

  # H0
  if(object$par$H0){
    cat("\n\n")
    #cat(paste0(rep("-",floor(options()$width/2)),collapse=""))
    cli_h2("\n H0: Under the null hypothesis\n")
    #cat(paste0(rep("-",floor(options()$width/2)),collapse=""))
    #
    cat("\n")
    cli_h3("Target parameters:\n")
    print(object$H0$target,row.names=FALSE)
    #
    if(object$extended>0){
      cat("\n")
      cli_h3("Efficacy:\n")
      print(object$H0$efficacy,row.names=FALSE)
      #
      cat("\n")
      cli_h3("Futility:\n")
      print(object$H0$futility,row.names=FALSE)
      #
      cat("\n")
      cli_h3("Sample size per group:\n")
      print(object$H0$summary.sample.sizes,row.names=FALSE)
      #cat("\n")
      #
      cat("\n")
      cli_h3("Scenarios:\n")
      print(object$H0$scenario,row.names=FALSE)
      cat(" where 0 = no stop, 1 = efficacy stop, 2 = futility stop\n")
      if(any(object$H0$scenario[,object$H0$target$id[-nrow(object$H0$target)+c(0:1)]]==3)){
        cat(",\n       3 = simultaneous efficacy and futility stops")
      }else{cat("\n")}
    }
  }
  # H1
  if(object$par$H1){
    cat("\n\n")
    #cat(paste0(rep("-",floor(options()$width/2)),collapse=""))
    cli_h2("\n H1: Under the alternative hypothesis\n")#cat("\n H1: Under the alternative hypothesis\n")
    #cat(paste0(rep("-",floor(options()$width/2)),collapse=""))
    #
    cat("\n")
    cli_h3("Target parameters:\n")
    print(object$H1$target,row.names=FALSE)
    #
    if(object$extended>0){
      cat("\n")
      cli_h3("Efficacy:\n")
      print(object$H1$efficacy,row.names=FALSE)
      #
      cat("\n")
      cli_h3("Futility:\n")
      print(object$H1$futility,row.names=FALSE)
      #
      cat("\n")
      cli_h3("Sample size per group:\n")
      print(object$H1$summary.sample.sizes,row.names=FALSE)
      #cat("\n")
      cat("\n")
      cli_h3("Scenarios:\n")
      print(object$H1$scenario,row.names=FALSE)
      cat(" where 0 = no stop, 1 = efficacy stop, 2 = futility stop")
      if(any(object$H1$scenario[,object$H1$target$id[-nrow(object$H1$target)+c(0:1)]]==3)){
        cat(",\n       3 = simultaneous efficacy and futility stops")
      }else{cat("\n")}
    }
  }
  cli_h1("")#cat(paste0(rep("-",options()$width),collapse=""))
  cat("\n")
}

# ----- ALLOCATION -----

#' @name alloc.simple
#' @title Simple allocation function
#' @description [alloc.simple] independently randomises each unit to a group (i.e., flips a coin for each unit) so that the observed allocation probabilities may be far from the target ones. This strategy is often considered to be a poor choice.
#' @param m the 'BATSS' ingredient '`m`', a scalar corresponding to the number of participants to be allocated.
#' @param prob the 'BATSS' ingredient '`prob`', a named vector of allocation ratios or probabilities.
#' @return [alloc.simple] returns an object of class \link[base]{factor} of length '`m`' with levels matching the names of the vector '`prob`'.
#' @seealso [alloc.balanced()], another group allocation function.
#' @examples
#' alloc.simple(100, prob = c(A=.4,B=.6))
#' table(alloc.simple(100, prob = c(A=.4,B=.6)))
#' table(alloc.simple(100, prob = c(A=.4,B=.6)))
#' @export
alloc.simple = function(m,prob){
  prob = abs(prob)/sum(abs(prob)) # normalise probabilities
  factor(sample(names(prob), m, replace=TRUE, prob = prob), levels=names(prob))
}

#' @name alloc.balanced
#' @title Balanced allocation function
#' @description [alloc.balanced] first allocates the largest possible number of units to the different groups given their exact target probabilities and then assigns randomly the remaining units to the different groups according to multinomial draws. This method leads to observed allocation probabilities matching the target ones when m*prob is an integer for each group and to observed allocation probabilities (on average) closer to the target ones compared to [alloc.simple].
#' @param m the 'BATSS' ingredient '`m`', a scalar corresponding to the number of participants to be allocated.
#' @param prob the 'BATSS' ingredient '`prob`', a named vector of allocation ratios or probabilities.
#' @return [alloc.balanced] returns an object of class \link[base]{factor} of length '`m`' with levels matching the names of the vector '`prob`'.
#' @seealso [alloc.simple()], another group allocation function.
#' @examples
#' alloc.balanced(100, prob = c(A=.4,B=.6))
#' table(alloc.balanced(100, prob = c(A=.4,B=.6)))
#' table(alloc.balanced(100, prob = c(A=.4,B=.6)))
#' @export
alloc.balanced = function(m,prob){
  prob = abs(prob)/sum(abs(prob))
  m0.g = floor(prob*m)
  m0   = sum(m0.g)
  factor(rep(names(prob),m0.g+rmultinom(1,m-m0,prob)),
         levels=names(prob))
}


# ----- EFFICACY, ARMS -----

#' @name eff.arm.simple
#' @title Simple arm efficacy stop 
#' @description allows stopping an arm for efficacy at a given look when the probability of the corresponding target parameter being greater or smaller (depending on the argument `'alternative'` of [batss.glm]) than `delta.eff` is greater than a fixed value `b`.
#' @param posterior the 'BATSS' ingredient '`posterior`' corresponding, in this context, to the (posterior) probability of the target parameter being greater or smaller (depending on the argument `'alternative'` of [batss.glm]) than '`delta.eff`'.
#' @param b the cut-off value used to declare efficacy (to be defined in `eff.arm.control`).
#' @return [eff.arm.simple] returns a logical constant.
#' @export
eff.arm.simple = function(posterior,b){
  posterior > b
}

#' @name eff.arm.infofract
#' @title information-fraction based arm efficacy stop 
#' @description allows stopping an arm for efficacy at a given look when the probability of the corresponding target parameter being greater or smaller (depending on the argument `'alternative'` of [batss.glm]) than `delta.eff` is greater than a function of the information fraction at that look.
#' @param posterior the 'BATSS' ingredient '`posterior`' corresponding, in this context, to the (posterior) probability of the target parameter being greater or smaller (depending on the argument `'alternative'` of [batss.glm]) than '`delta.eff`'.
#' @param b a tuning parameter (to be defined in `eff.arm.control`).
#' @param n the 'BATSS' ingredient '`n`' corresponding to the vector of number of recruited participants per arm including the control group.
#' @param N the 'BATSS' ingredient '`N' corresponding to the maximum (planned) sample size.
#' @param p a tuning parameter (to be defined in `eff.arm.control`).
#' @return [eff.arm.infofract] returns a logical constant.
#' @export
eff.arm.infofract = function(posterior,b,n,N,p){
  posterior > (1-(b*(sum(n)/N)^p))
}


# ----- EFFICACY, TRIAL -----

#' @name eff.trial.all
#' @title trial efficacy stop 
#' @description allows stopping the trial for efficacy if *all* target parameters reached efficacy at the look of interest or before.
#' @param eff.target the 'BATSS' ingredient '`eff.target`' corresponding to a \link[base]{logical} vector of the same length as argument `which` (i.e., the number of target parameters) indicating if efficacy was reached for each target parameter at that stage or at a previous stage.
#' @return [eff.trial.all] returns a logical constant.
#' @export
eff.trial.all = function(eff.target){all(eff.target)}

#' @name eff.trial.any
#' @title trial efficacy stop 
#' @description allows stopping the trial for efficacy if *at least one* target parameter reached efficacy at the look of interest.
#' @param eff.target the 'BATSS' ingredient '`eff.target`' corresponding to a \link[base]{logical} vector of the same length as argument `which` (i.e., the number of target parameters) indicating if efficacy was reached for each target parameter at that stage or at a previous stage.
#' @return [eff.trial.any] returns a logical constant.
#' @export
eff.trial.any = function(eff.target){any(eff.target)}


# ----- FUTILITY, ARMS -----

#' @name fut.arm.simple
#' @title arm futility stop 
#' @description allows stopping an arm for futility when the probability of the corresponding target parameter being greater or smaller (depending on the argument `'alternative'` of [batss.glm]) than '`delta.fut`' is smaller than a fixed value '`b`'
#' @param posterior the 'BATSS' ingredient '`posterior`' corresponding, in this context, to the (posterior) probability of the target parameter being greater or smaller (depending on the argument `'alternative'` of [batss.glm]) than '`delta.fut`'.
#' @param b the cut-off value used to declare futility (to be defined in `fut.arm.control`).
#' @return [fut.arm.simple] returns a logical constant.
#' @export
fut.arm.simple = function(posterior,b){
  posterior < b
}


# ----- FUTILITY, TRIAL -----

#' @name fut.trial.all
#' @title trial futility stop 
#' @description allows stopping the trial for efficacy if *all* active treatment reached futility at the look of interest or before.
#' @param fut.target the 'BATSS' ingredient '`fut.target`' corresponding to a \link[base]{logical} vector of the same length as argument `which` (i.e., the number of target parameters) indicating if futility was declared for each target parameter at that stage or at a previous stage.
#' @return [fut.trial.all] returns a logical constant.
#' @export
fut.trial.all = function(fut.target){all(fut.target)}


# ----- RAR -----

#' @name RAR.trippa
#' @title RAR of Trippa et al. (2012)
#' @description define the group allocation probabilities based on the response adaptive randomisation rule of Trippa et al. (2012)
#' @param posterior the 'BATSS' ingredient '`posterior`' corresponding, in this context, to the (posterior) probability of the active target parameters being greater or smaller (depending on the argument `'alternative'` of [batss.glm]) than '`delta.RAR`'.
#' @param n the 'BATSS' ingredient '`n`' corresponding to the vector of number of recruited participants per arm including the control group at the look of interest.
#' @param N the 'BATSS' ingredient '`N' corresponding to the maximum (planned) sample size.
#' @param ref the 'BATSS' ingredient '`ref`' corresponding to a \link[base]{logical} vector of the same length and order as `'prob0'` (i.e., number of arms initially included in the study including the reference group)) and indicating which group is the reference one.
#' @param active the 'BATSS' ingredient '`active`' corresponding to a \link[base]{logical} vector of the same length and order as `'prob0'` (i.e., number of arms initially included in the study including the reference group)) and  indicating if each arm is active at the look of interest.
#' @param gamma a scaling factor (to be defined in `RAR.arm.control`).
#' @param eta a scaling factor (to be defined in `RAR.arm.control`).
#' @param nu a scaling factor (to be defined in `RAR.arm.control`).
#' @return [RAR.trippa] returns a vector of probabilities with length of active.
#' @export
RAR.trippa = function(posterior,n,N,ref,active,gamma,eta,nu){
  g = sum(active)
  h = gamma*(sum(n)/N)^eta
  prob = rep(NA,g)

  # reference
  prob[1] = (exp(max(n[!ref])-n[ref])^nu)/(g-1)
  # targets
  prob[2:g] = (posterior^h)/(sum(posterior^h))
  #
  prob
}

#' @name RAR.optimal
#' @title 'Optimal' control allocation
#' @description technically not response adaptive but keeps allocation ratio to control at the square root of active intervention arms
#' @param active the 'BATSS' ingredient '`active`' corresponding to a \link[base]{logical} vector of the same length and order as `'prob0'` (i.e., number of arms initially included in the study including the reference group)) and  indicating if each arm is active at the look of interest.
#' @return [RAR.optimal] returns a vector of probabilities with length of active.
#' @export
RAR.optimal = function(active){
  K    = sum(active)-1
  tot  = K+sqrt(K)
  prob = c(sqrt(K), rep(1,K))/tot
  prob
}

#' @name print.batss
#' @title Print function for BATSS outputs
#' @description Print method function for objects of class 'batss' (i.e., output of the function [batss.glm]).
#' @param x An object of class 'batss'.
#' @param ... Additional arguments affecting the print produced.
#' @seealso [batss.glm()], the function generating S3 objects of class 'batss'. 
#' @returns Prints information for objects of class 'batss'.
#' @export
print.batss = function(x, ...){
    cat("\n Bayesian adaptive trial Monte Carlo simulation.\n")
    if(!is.null(x$par$RAR)){
        cat(" with response adapative randomisation (",
            x$call$RAR,
            ")\n",sep="")
    }
    cat(" (",length(x$par$seed)," Monte Carlo samples)\n",sep="")
    cat("\n Variables:\n")
    for(i in 2:length(x$call$var)){
        cat("  *",names(x$call$var)[i],":",as.character(x$call$var)[i],"\n")
    }
    if(!is.null(x$par$RAR)){
        cat("\nGroup randomisation:\n")
        cat("  *",x$call$RAR,"\n")
    }
    cat("\n Model:\n")
    cat("  *",format(x$call$model),if (x$type=="surv") "\n" else paste("(with",ifelse(is.null(x$call$link),"identity",x$call$link), "link)\n"))
    cat("\n Call:\n ")
    print(x$call)
}

#' @name batss.combine
#' @title Combines outputs generated by [batss.glm]
#' @description Combines different evaluations of [batss.glm] considering the same trial design but different sets of seeds. This function is useful when the evaluation of Monte Carlo samples generated by different seeds was split in sets computed by different nodes/cpus. The output of this function is of class 'batss' meaning that the usual generic functions (print, summary, plot) can be used.
#' @param paths Vector indicating the paths to the rdata files containing the outputs of the function [batss.glm] considering the same trial design but different set of seeds. This requires the argument '`extended`' of the function [batss.glm] to be > 0. 
#' @param force a \link[base]{logical} with default `force=FALSE`. Among other checks, [batss.glm] controls that the \link[base]{call}s of the Monte Carlo trials to be combined are \link[base]{identical} and stops if they are not (Note that this check is not bullet proof: such a check, for example, would be able to note that two sets of Monte Carlo trials used a `eff.arm` function named the same way and considered the same optional parameters but would be blind to the fact that they could correspond to two *different* functions). `force=TRUE` forces [batss.glm] to ignore this check. This could be useful if the calls differ due to the `batss` objects to be combined being generated using different versions of [batss.glm].
#' @returns an object of class 'batss'. Refer to the section 'Value' in [batss.glm] for details about this object structure. 
#' @seealso [batss.glm()], the function allowing to simulate Bayesian adaptive trials with GLM endpoint for different seeds. 
#' @export
batss.combine = function(paths, force=FALSE){# paths
    ##
    ## load results 
    ##
    objlist = as.list(rep(NA,length(paths)))
    for(ow in 1:length(paths)){# ow=2
        loaded = load(paths[ow])
        if(length(loaded)==1){
            if(!inherits(get(loaded),"batss")){stop("loaded object not of class 'batss'")}
        }else{
            if(length(loaded)>1){
                class.obj = rep(NA,length(loaded))
                for(oww in 1:length(loaded)){
                    class.obj[oww] = inherits(get(loaded[oww]),"batss")
                }
                loaded = loaded[class.obj][1]
            }
        }
        objlist[[ow]]  = get(loaded)
    }
    
    ##
    ## check calls
    ##
    tmp   = lapply(objlist,function(x)as.list(x$call)[names(as.list(x$call))!="R"])
    check = all(sapply(tmp,function(x,ref){identical(x,ref)},ref=tmp[[1]]))
    if(!check){
        if(!force){
            stop("the calls that generated the objects to be combined are different")
        }else{
            warning("the calls that generated the objects to be combined are different")
        }
    }

    ##
    ## combine results
    ##
    out     = objlist[[1]]
    if(is.null(out$call$extended)|(out$call$extended<1)){stop("argument 'extended' of the loaded object is NULL or equal to 0")}
    n.look  = nrow(out$look)
    # seeds
    if(length(objlist)>1){
        for(ow in 2:length(objlist)){
            out$par$seed = c(out$par$seed,objlist[[ow]]$par$seed)
        }
    }
    if(any(table(out$par$seed)>1)){stop("duplicated seeds")}
    # H0 and H1 trials
    Hw      = c("H0","H1")[c(objlist[[1]]$par$H0,objlist[[1]]$par$H1)]
    for(hw in 1:length(Hw)){# hw=1
        n.obj   = length(objlist)
        id.target = objlist[[1]][[Hw[hw]]]$trial[[1]]$target
        estimate  = objlist[[1]][[Hw[hw]]]$estimate
        trial_r   = objlist[[1]][[Hw[hw]]]$trial
        if(length(objlist)>1){
            for(ow in 2:n.obj){
                if(is.null(objlist[[ow]]$call$extended)|(objlist[[ow]]$call$extended<1)){
                        stop("argument 'extended' of the loaded object is NULL or equal to 0")
                    }            
                estimate = abind::abind(estimate,objlist[[ow]][[Hw[hw]]]$estimate,along=3)
                trial_r  = c(trial_r,objlist[[ow]][[Hw[hw]]]$trial)
            }
            }
        tar.p  = batss.res.tp(estimate,id.target)
        tar.g  = batss.res.tg(estimate,id.target)
        eff.p  = batss.res.ep(estimate,id.target,n.look)
        eff.g  = batss.res.eg(estimate,id.target,n.look)
        fut.p  = batss.res.fp(estimate,id.target,n.look)
        fut.g  = batss.res.fg(estimate,id.target,n.look)
        sample = batss.res.s1(trial_r,group=out$par$group$id,
                              type=c(apply(estimate[,"type",,drop=FALSE],2:3,paste0,collapse="")),
                              early=c(apply(estimate[,"look",,drop=FALSE]<n.look,2:3,all)))
        scenario = batss.res.s2(sample,target=id.target$id)
        out[[Hw[hw]]] = list(estimate = estimate,
                        target   = list(par=tar.p,global=tar.g),
                        efficacy = list(par=eff.p,global=eff.g),
                        futility = list(par=fut.p,global=fut.g),
                        sample=sample,scenario=scenario)
        if(!is.null(out$call$extended)){
            if(out$call$extended>0){
                out[[Hw[hw]]]$trial = trial_r
            }       
        }
    }
    out
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glm_combine.R
\name{batss.combine}
\alias{batss.combine}
\title{Combines outputs generated by \link{batss.glm}}
\usage{
batss.combine(paths, force = FALSE)
}
\arguments{
\item{paths}{Vector indicating the paths to the rdata files containing the outputs of the function \link{batss.glm} considering the same trial design but different set of seeds. This requires the argument '\code{extended}' of the function \link{batss.glm} to be > 0.}

\item{force}{a \link[base]{logical} with default \code{force=FALSE}. Among other checks, \link{batss.glm} controls that the \link[base]{call}s of the Monte Carlo trials to be combined are \link[base]{identical} and stops if they are not (Note that this check is not bullet proof: such a check, for example, would be able to note that two sets of Monte Carlo trials used a \code{eff.arm} function named the same way and considered the same optional parameters but would be blind to the fact that they could correspond to two \emph{different} functions). \code{force=TRUE} forces \link{batss.glm} to ignore this check. This could be useful if the calls differ due to the \code{batss} objects to be combined being generated using different versions of \link{batss.glm}.}
}
\value{
an object of class 'batss'. Refer to the section 'Value' in \link{batss.glm} for details about this object structure.
}
\description{
Combines different evaluations of \link{batss.glm} considering the same trial design but different sets of seeds. This function is useful when the evaluation of Monte Carlo samples generated by different seeds was split in sets computed by different nodes/cpus. The output of this function is of class 'batss' meaning that the usual generic functions (print, summary, plot) can be used.
}
\seealso{
\code{\link[=batss.glm]{batss.glm()}}, the function allowing to simulate Bayesian adaptive trials with GLM endpoint for different seeds.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_functions.R
\name{RAR.trippa}
\alias{RAR.trippa}
\title{RAR of Trippa et al. (2012)}
\usage{
RAR.trippa(posterior, n, N, ref, active, gamma, eta, nu)
}
\arguments{
\item{posterior}{the 'BATSS' ingredient '\code{posterior}' corresponding, in this context, to the (posterior) probability of the active target parameters being greater or smaller (depending on the argument \code{'alternative'} of \link{batss.glm}) than '\code{delta.RAR}'.}

\item{n}{the 'BATSS' ingredient '\code{n}' corresponding to the vector of number of recruited participants per arm including the control group at the look of interest.}

\item{N}{the 'BATSS' ingredient '`N' corresponding to the maximum (planned) sample size.}

\item{ref}{the 'BATSS' ingredient '\code{ref}' corresponding to a \link[base]{logical} vector of the same length and order as \code{'prob0'} (i.e., number of arms initially included in the study including the reference group)) and indicating which group is the reference one.}

\item{active}{the 'BATSS' ingredient '\code{active}' corresponding to a \link[base]{logical} vector of the same length and order as \code{'prob0'} (i.e., number of arms initially included in the study including the reference group)) and  indicating if each arm is active at the look of interest.}

\item{gamma}{a scaling factor (to be defined in \code{RAR.arm.control}).}

\item{eta}{a scaling factor (to be defined in \code{RAR.arm.control}).}

\item{nu}{a scaling factor (to be defined in \code{RAR.arm.control}).}
}
\value{
\link{RAR.trippa} returns a vector of probabilities with length of active.
}
\description{
define the group allocation probabilities based on the response adaptive randomisation rule of Trippa et al. (2012)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_functions.R
\name{alloc.balanced}
\alias{alloc.balanced}
\title{Balanced allocation function}
\usage{
alloc.balanced(m, prob)
}
\arguments{
\item{m}{the 'BATSS' ingredient '\code{m}', a scalar corresponding to the number of participants to be allocated.}

\item{prob}{the 'BATSS' ingredient '\code{prob}', a named vector of allocation ratios or probabilities.}
}
\value{
\link{alloc.balanced} returns an object of class \link[base]{factor} of length '\code{m}' with levels matching the names of the vector '\code{prob}'.
}
\description{
\link{alloc.balanced} first allocates the largest possible number of units to the different groups given their exact target probabilities and then assigns randomly the remaining units to the different groups according to multinomial draws. This method leads to observed allocation probabilities matching the target ones when m*prob is an integer for each group and to observed allocation probabilities (on average) closer to the target ones compared to \link{alloc.simple}.
}
\examples{
alloc.balanced(100, prob = c(A=.4,B=.6))
table(alloc.balanced(100, prob = c(A=.4,B=.6)))
table(alloc.balanced(100, prob = c(A=.4,B=.6)))
}
\seealso{
\code{\link[=alloc.simple]{alloc.simple()}}, another group allocation function.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glm_generic.R
\name{print.batss}
\alias{print.batss}
\title{Print function for BATSS outputs}
\usage{
\method{print}{batss}(x, ...)
}
\arguments{
\item{x}{An object of class 'batss'.}

\item{...}{Additional arguments affecting the print produced.}
}
\value{
Prints information for objects of class 'batss'.
}
\description{
Print method function for objects of class 'batss' (i.e., output of the function \link{batss.glm}).
}
\seealso{
\code{\link[=batss.glm]{batss.glm()}}, the function generating S3 objects of class 'batss'.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glm.R
\name{batss.glm}
\alias{batss.glm}
\title{Bayesian adaptive trial simulations for generalised linear models}
\usage{
batss.glm(
  model,
  var,
  var.control = NULL,
  family = "gaussian",
  link = "identity",
  beta,
  which,
  alternative = "greater",
  R = 10000,
  N,
  interim,
  prob0,
  delta.eff = 0,
  delta.fut = delta.eff,
  delta.RAR = 0,
  eff.arm,
  eff.arm.control = NULL,
  eff.trial = NULL,
  eff.trial.control = NULL,
  fut.arm,
  fut.arm.control = NULL,
  fut.trial = NULL,
  fut.trial.control = NULL,
  RAR = NULL,
  RAR.control = NULL,
  H0 = TRUE,
  computation = "parallel",
  mc.cores = getOption("mc.cores", 3L),
  extended = 0,
  ...
)
}
\arguments{
\item{model}{an object of class '\link[stats]{formula}' indicating a symbolic description of the model to be fitted (as in the \link[stats]{lm} and \link[stats]{glm} functions).}

\item{var}{A list. Each entry corresponds to a variable described under '\code{model}' and indicates the name of a function allowing to generate variates (like \link[stats]{rnorm} and \link[stats]{rexp}, for example). The list names have to match the variable names unded in '\code{model}' and its first element should correspond to the model outcome. The grouping variable corresponding to the target parameters has to be of class '\link[base]{factor}' with levels corresponding to the names indicated in argument \code{prob0} (see below).}

\item{var.control}{An optional list of control parameters for the functions indicated in '\code{var}'. The names of the list items need to correspond to the names used in '\code{var}'. Each element is another list with names of the elements corresponding to the parameter names of the functions specified in '\code{var}'.}

\item{family}{A character string indicating the name of the conditional distribution as described in the package INLA (check \link[INLA]{inla.list.models}). Default set to '\code{gaussian}'.}

\item{link}{A character string describing the link function to be used in the model to relate the outcome to the set of predictors: 'identity', 'log', 'logit', 'probit', 'robit', 'cauchit', 'loglog' and 'cloglog' are the currently available options. Default set to 'identity'.}

\item{beta}{A numerical vector of parameter values for the linear predictor. Its length has to match the number of column of the \strong{X} matrix induced by the formula indicated under '\code{model}' (check \link[stats]{model.matrix}).}

\item{which}{A numerical vector indicating the position of the target \code{beta} parameters.}

\item{alternative}{A vector of strings providing the one-sided direction of the alternative hypothesis corresponding to each target parameter indicated under '\code{which}' (in the same order). Possibilities are 'greater' (default) or 'less'. If the vector is of length 1, the same direction will be used for all target parameter tests.}

\item{R}{a vector of natural numbers to be used as seeds (check \link[base]{set.seed}) for the different Monte Carlo trials (the vector length will thus correspond to the number of Monte Carlo trials). When \code{R} is a scalar, seeds \code{1} to \code{R} are used, where \code{R} corresponds to the number of Monte Carlo trials.}

\item{N}{A scalar indicating the maximum sample size.}

\item{interim}{A list of parameters related to interim analyses. Currently, only '\code{recruited}' is available.  It consists in a vector of integers indicating the number of completed observations at each look, last excluded, in increasing order. Setting \code{interim = NA} specifies a fixed (non-adaptive) design with a single look at the maximum sample size \code{N}.}

\item{prob0}{A named vector with initial allocation probabilities. Names need to correspond to the levels of the grouping variable. If \code{RAR = NULL}, these probabilities/ratios will be used throughout (fixed allocation probabilities).}

\item{delta.eff}{A vector (of length equal to the number of looks (i.e., number of interims + 1)) of clinically meaningful treatment effect values (on the linear predictor scale) to be used to define the efficacy-related posterior probabilities for each target parameter at each look. If a scalar is provided, the same value is used at each look. The default is \code{delta.eff = 0}.}

\item{delta.fut}{A vector (of length equal to the number of looks (i.e., number of interims + 1)) of clinically meaningful treatment effect values (on the linear predictor scale) to be used to define the futility-related posterior probabilities for each target parameter at each look. If a scalar is provided, the same value is used at each look. The default is \code{delta.fut = delta.eff}.}

\item{delta.RAR}{A vector (of length equal to the number of looks (i.e., number of interims + 1)) of clinically meaningful treatment effect values (on the linear predictor scale) to be used to define the RAR-related posterior probabilities for each target parameter at each look. If a scalar is provided, the same value is used at each interim analysis. The default is \code{delta.RAR = 0}. Note that, when a vector is provided, its last value is ignored as no randomisation is made at the last look.}

\item{eff.arm}{A function defining if efficacy has been achieved at a given look given the information available at that stage a given target parameter. The output of this function must be a \link[base]{logical} (of length 1). Arguments of this function will typically consider 'BATSS' ingredients. Check \link{eff.arm.simple} and \link{eff.arm.infofract} for examples.}

\item{eff.arm.control}{An optional list of parameters for the function indicated in '\code{eff.arm}'.}

\item{eff.trial}{A function defining if the trial can be stopped for efficacy given the output of the function indicated in '\code{eff.arm}'. The output of this function must be a \link[base]{logical} of length one. Arguments of this function will typically only consider the 'BATSS' ingredient \code{eff.target}. Check \link{eff.trial.all} and \link{eff.trial.any} for examples. When \code{eff.trial = NULL} (default), the trial stops for efficacy when \emph{all} target parameters are found to be effective (like in \link{eff.trial.all}).}

\item{eff.trial.control}{An optional list of parameters for the function indicated in '\code{eff.trial}'.}

\item{fut.arm}{A function defining if futility has been achieved at a given look given the information available at that stage for each target parameter. The output of this function must be a \link[base]{logical} (of length 1). Arguments of this function will typically consider 'BATSS' ingredients. Check \link{fut.arm.simple} to see an example of such a function.}

\item{fut.arm.control}{An optional list of parameters for the function indicated in '\code{fut.arm}'.}

\item{fut.trial}{A function defining if the trial can be stopped for futility given the output of the function indicated in '\code{fut.arm}'. The output of this function must be a \link[base]{logical} of length one. Arguments of this function will typically only consider the 'BATSS' ingredient \code{fut.target}. Check \link{fut.trial.all} for an example of such a function. When \code{fut.trial = NULL} (default), the trial stops for futility when \emph{all} target parameters are found to be futile (like in \link{fut.trial.all}).}

\item{fut.trial.control}{An optional list of parameters for the function indicated in '\code{fut.trial}'.}

\item{RAR}{A function defining the response-adaptive randomisation probabilities of each group - reference group included - with the same group names and ordering as used in '\code{prob0}'. Arguments of this function will typically consider 'BATSS' ingredients. Check \link{RAR.trippa} and \link{RAR.optimal} for examples. If \code{RAR = NULL} (default), the probabilities/ratios indicated under \code{prob0} will be used throughout (fixed allocation probabilities).}

\item{RAR.control}{An optional list of control parameters for the function provided in '\code{RAR}'.}

\item{H0}{A logical indicating whether the simulation should also consider the case with all target parameters set to 0 to check the probability of rejecting the hypothesis that the target parameter value is equal to 0 individually (pairwise type I error) or globally (family-wise error rate). Default set to \code{H0=TRUE}.}

\item{computation}{A character string indicating how the computation should be performed. Possibilities are 'parallel' or 'sequential' with default \code{computation="parallel"} meaning that the computation is split between \code{mc.cores}.}

\item{mc.cores}{An integer indicating the number of CPUs to be used when \code{computation="parallel"} (Default to 3 if no global '\code{mc.cores}' global option is available via \link[base]{getOption}).}

\item{extended}{an integer indicating the type of results to be returned. 0 (default) provides summary statistics, 1 adds the results of each Monte Carlo trial and 2 additionally returns each Monte Carlo dataset. \link{batss.combine} requires extended > 0 as the function needs to merge results of different sets of seeds.}

\item{...}{Additional arguments to control fitting in \link[INLA]{inla}.}
}
\value{
The function \link{batss.glm} returns an S3 object of class 'batss' with available print/summary/plot functions
\itemize{
\item beta - A data frame providing information related to the beta parameter vector, like parameter names and values, for example.
\item look - A data frame providing information related to looks, like sample size of a given interim (m) and cumulative sample size at a given interim (n), for example.
\item par - A list providing different information, like the used seeds (seed) and the groups (group), for example.
\item H1 - A list providing trial results under the alternative, like the estimates per target parameter when the corresponding arm was stopped (estimate), the efficacy and futility probabilites per target parameter and overall (target, efficacy and futility), the sample size per group and trial (sample), the probabilities associated to each combination of efficacy and futility per group (scenario), the detailed results per trial (trial), for example.
\item H0 - A list providing trial results under the global null hypothesis (same structure as H1).
\item call - The matched call.
\item type - The type of 'BATSS' analysis (only 'glm' is currently available).
}
}
\description{
Simulation of Bayesian adaptive trials with GLM endpoint using Integrated Nested Laplace Approximation (INLA).
}
\examples{
\donttest{
# Example: 
# * Gaussian conditional distribution with sigma = 5
# * 3 groups with group means 'C' = 1 (ref), 'T1' = 2, 'T2' = 3,
#     where higher means correspond to better outcomes 
# * 5 interim analyses occurring when n = 100, 120, 140, 160, and 180
# * fixed and equal allocation probabilities per arm (i.e., no RAR)
# * max sample size = 200 
# * efficacy stop per arm when the prob of the corresponding parameter 
#     being greater than 0 is greater than 0.975 (?eff.arm.simple)
# * futility stop per arm when the prob of the corresponding parameter 
#     being greater than 0 is smaller than 0.05 (?fut.arm.simple) 
# * trial stop once all arms have stopped (?eff.trial.all and ?fut.trial.all)
#     or the max sample size was reached 

sim = batss.glm(model            = y ~ group,   
                var              = list(y     = rnorm,
                                        group = alloc.balanced),
                var.control      = list(y = list(sd = 5)),
                beta             = c(1, 1, 2),
                which            = c(2:3),
                alternative      = "greater",
                R                = 20,
                N                = 200,
                interim          = list(recruited = seq(100, 180, 20)),
                prob0            = c(C = 1/3, T1 = 1/3, T2 = 1/3),
                eff.arm          = eff.arm.simple,
                eff.arm.control  = list(b = 0.975),
                fut.arm          = fut.arm.simple,
                fut.arm.control  = list(b = 0.05),
                computation      = "parallel",
                H0               = TRUE,
                mc.cores         = 2)# better: parallel::detectCores()-1
}
}
\seealso{
\link{summary.batss} and \link{plot.batss} for detailed summaries and plots, and \link{batss.combine} to combine different evaluations of \link{batss.glm} considering the same trial design but different sets of seeds (useful for cluster computation).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_functions.R
\name{eff.arm.infofract}
\alias{eff.arm.infofract}
\title{information-fraction based arm efficacy stop}
\usage{
eff.arm.infofract(posterior, b, n, N, p)
}
\arguments{
\item{posterior}{the 'BATSS' ingredient '\code{posterior}' corresponding, in this context, to the (posterior) probability of the target parameter being greater or smaller (depending on the argument \code{'alternative'} of \link{batss.glm}) than '\code{delta.eff}'.}

\item{b}{a tuning parameter (to be defined in \code{eff.arm.control}).}

\item{n}{the 'BATSS' ingredient '\code{n}' corresponding to the vector of number of recruited participants per arm including the control group.}

\item{N}{the 'BATSS' ingredient '`N' corresponding to the maximum (planned) sample size.}

\item{p}{a tuning parameter (to be defined in \code{eff.arm.control}).}
}
\value{
\link{eff.arm.infofract} returns a logical constant.
}
\description{
allows stopping an arm for efficacy at a given look when the probability of the corresponding target parameter being greater or smaller (depending on the argument \code{'alternative'} of \link{batss.glm}) than \code{delta.eff} is greater than a function of the information fraction at that look.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_functions.R
\name{RAR.optimal}
\alias{RAR.optimal}
\title{'Optimal' control allocation}
\usage{
RAR.optimal(active)
}
\arguments{
\item{active}{the 'BATSS' ingredient '\code{active}' corresponding to a \link[base]{logical} vector of the same length and order as \code{'prob0'} (i.e., number of arms initially included in the study including the reference group)) and  indicating if each arm is active at the look of interest.}
}
\value{
\link{RAR.optimal} returns a vector of probabilities with length of active.
}
\description{
technically not response adaptive but keeps allocation ratio to control at the square root of active intervention arms
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.batss.R
\name{summary.batss}
\alias{summary.batss}
\title{Summary function for 'BATSS' outputs}
\usage{
\method{summary}{batss}(object, extended = NULL, ...)
}
\arguments{
\item{object}{An object of class 'batss' (i.e., output of the function \link{batss.glm}).}

\item{extended}{A logical indicating if a standard (extended = FALSE, default) or extended output (extended = TRUE) should be returned. Default to \code{NULL} in which case the input of the argument \code{extended} chosen when generating \code{object} with \code{\link[=batss.glm]{batss.glm()}} is used.}

\item{...}{Additional arguments affecting the summary produced.}
}
\value{
Object of class 'summary.batss'.

The function \link{summary.batss} returns an S3 list of class 'summary.batss' with available print functions. The list elements are
\itemize{
\item beta - A data frame providing information related to the beta parameter vector, such as parameter names and values, for example.
\item look - A data frame providing information related to looks, like sample size of a given interim (m) and cumulative sample size at a given interim (n), for example.
\item par - A list providing different information, like the used seeds (seed) and the groups (group), for example.
\item H1 - A list providing trial aggregated results under the alternative, like the probability of efficacy, futility, or both, per arm or globally (\code{object$H1$target}), the probability of stopping early for efficacy (\code{object$H1$efficacy}) and futility (\code{object$H1$futility}), the sample size expectation, standard deviation, and quantiles 0.1, 0.5 and 0.9, per group and overall (\code{object$H1$summary.sample.sizes}), the probabilities associated to each combination of efficacy and futility per group (scenario).
\item H0 - A list providing trial aggregated results under the global null hypothesis (same structure as H1).
\item call - The matched call.
\item type - The type of 'BATSS' analysis (only 'glm' is currently available).
}
}
\description{
Summary method function for objects of class 'batss'.
}
\seealso{
\code{\link[=batss.glm]{batss.glm()}}, the function generating S3 objects of class 'batss'.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_functions.R
\name{alloc.simple}
\alias{alloc.simple}
\title{Simple allocation function}
\usage{
alloc.simple(m, prob)
}
\arguments{
\item{m}{the 'BATSS' ingredient '\code{m}', a scalar corresponding to the number of participants to be allocated.}

\item{prob}{the 'BATSS' ingredient '\code{prob}', a named vector of allocation ratios or probabilities.}
}
\value{
\link{alloc.simple} returns an object of class \link[base]{factor} of length '\code{m}' with levels matching the names of the vector '\code{prob}'.
}
\description{
\link{alloc.simple} independently randomises each unit to a group (i.e., flips a coin for each unit) so that the observed allocation probabilities may be far from the target ones. This strategy is often considered to be a poor choice.
}
\examples{
alloc.simple(100, prob = c(A=.4,B=.6))
table(alloc.simple(100, prob = c(A=.4,B=.6)))
table(alloc.simple(100, prob = c(A=.4,B=.6)))
}
\seealso{
\code{\link[=alloc.balanced]{alloc.balanced()}}, another group allocation function.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_functions.R
\name{fut.arm.simple}
\alias{fut.arm.simple}
\title{arm futility stop}
\usage{
fut.arm.simple(posterior, b)
}
\arguments{
\item{posterior}{the 'BATSS' ingredient '\code{posterior}' corresponding, in this context, to the (posterior) probability of the target parameter being greater or smaller (depending on the argument \code{'alternative'} of \link{batss.glm}) than '\code{delta.fut}'.}

\item{b}{the cut-off value used to declare futility (to be defined in \code{fut.arm.control}).}
}
\value{
\link{fut.arm.simple} returns a logical constant.
}
\description{
allows stopping an arm for futility when the probability of the corresponding target parameter being greater or smaller (depending on the argument \code{'alternative'} of \link{batss.glm}) than '\code{delta.fut}' is smaller than a fixed value '\code{b}'
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_functions.R
\name{fut.trial.all}
\alias{fut.trial.all}
\title{trial futility stop}
\usage{
fut.trial.all(fut.target)
}
\arguments{
\item{fut.target}{the 'BATSS' ingredient '\code{fut.target}' corresponding to a \link[base]{logical} vector of the same length as argument \code{which} (i.e., the number of target parameters) indicating if futility was declared for each target parameter at that stage or at a previous stage.}
}
\value{
\link{fut.trial.all} returns a logical constant.
}
\description{
allows stopping the trial for efficacy if \emph{all} active treatment reached futility at the look of interest or before.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.batss.R
\name{plot.batss}
\alias{plot.batss}
\title{Plot function for 'BATSS' outputs}
\usage{
\method{plot}{batss}(
  x,
  which = 1:4,
  ask = TRUE,
  hypothesis = "H1",
  title = TRUE,
  legend = TRUE,
  ess = TRUE,
  percentage = TRUE,
  beta = TRUE,
  col = c("#008B0040", "#8B3A3A40", "#8B897040", "#FF990075"),
  smooth = 1,
  ...
)
}
\arguments{
\item{x}{An object of class 'batss' (i.e., output of the function \link{batss.glm}).}

\item{which}{An integer vector indicating the list of desired plots. If a subset of the plots is required, specify a subset of the numbers \code{1:4}. By default, all plots are provided, i.e., \code{which=1:4}. Plot \code{1} displays the boxplot of the total sample size as well as the boxplot of the sample sizes per group, Plot \code{2} displays a barplot of the probability of stopping at each look, plot \code{3} displays the violin plot of the sample size per group, and plot \code{4} displays the Monte Carlo trial target estimates as a function of the sample size.}

\item{ask}{A logical. If \code{TRUE}, the user is prompted to hit the \kbd{Enter} key before each plot. See \code{\link{par}(ask=.)}. The default is \code{ask=TRUE}.}

\item{hypothesis}{A character string indicating which alternative hypothesis to use for analyses considering both "H0" and "H1", with options "H1" (default) and "H0".}

\item{title}{Either a \link[base]{logical} indicating if a title should be added or a string (of class \link[base]{character}) indicating the title to be added. If \code{title} equals \code{TRUE} (default), the title 'Under 'H1' or 'Under 'H0' (depending on the argument \code{hypothesis}) is added to the outer margin of the plot. No outer margin space is added if \code{title = FALSE}.}

\item{legend}{A \link[base]{logical} (with default set to \code{TRUE}) if a legend should be added at the bottom of plots 3 and 4, or a list with names \code{height}, \code{cex} and \code{pt} respectively indicating i/ the fraction of the plot to be used for the legend as a numeric (default is \code{.15}), ii/ the character expansion factor relative to current \code{par("cex")} as a numeric (default to \code{1.25}), and iii/ the expansion factor(s) for the points as a numeric (default to \code{2}). The input \code{legend = TRUE} is equivalent to \code{legend = c(height=.15, cex=1, pt=2)}.}

\item{ess}{A \link[base]{logical} (with default set to \code{TRUE}) indicating if the expected sample size should be displayed in plots 2, 3 and 4, or a list with names \code{col}, \code{cex} and \code{bg} respectively indicating i/ the colour of the label as a character (plots 2, 3, and 4), ii/ the text expension level as a numerical value (plots 2, 3 and 4) and iii/ the background colour as a character (plot 3). The input \code{ess = TRUE} is equivalent to \code{ess = list(col="blue", cex=1, bg="#FFD70070")}.}

\item{percentage}{A \link[base]{logical} (with default set to \code{TRUE}) indicating if the probability of stopping at each look should be displayed in plots 2 (as a percentage), or a list with names \code{col}, and \code{cex} indicating i/ the colour of the label as a character, ii/ the text expension level as a numerical value. The input \code{percentage = TRUE} is equivalent to \code{percentage = list(col="violet", cex=1)}.}

\item{beta}{A \link[base]{logical} (with default set to \code{TRUE}) indicating if the assumed target parameter value(s) should be displayed in plot 4, or a list with names \code{col}, \code{lwd}, and \code{lty} respectively indicating i/ the colour of the target parameter value horizontal line(s) as a character, ii/ the thickness of the line(s) as a numerical value  and iii/ the type of line as an integer (see \link[graphics]{par} for details). The input \code{beta = TRUE} is equivalent to \code{beta = list(col="blue", lwd=1.5, lty=1)}.}

\item{col}{A vector of length 4 specifying the colours to be used. Default to \code{c("#8B897040","#008B0040","#8B3A3A40","#FF990075")} where the last two digits of the hexadecimal strings specify the level of transparency. Refer to the Section 'colour specification' in \link[graphics]{par} for details. If the length of \code{col} equals 1, the same colour is used for all cases. For plots \code{1} and \code{2}, the 3rd colour of the vector \code{col} is used to display the barplot and boxplots.}

\item{smooth}{A numerical (>0) indictating the level of smoothing of the violin plots (for plot \code{3}). Default to \code{1}. When \code{smooth=NULL}, the smoothing value is optimised in the \link[sm]{sm.density} function.}

\item{...}{Additional arguments affecting the plot produced, like ylim and ylab.}
}
\value{
Generates graphical displays of results for objects of class 'batss'.
}
\description{
Plot for objects of class 'batss'
}
\seealso{
\code{\link[=batss.glm]{batss.glm()}}, the function generating S3 objects of class 'batss'.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.batss.R
\name{print.summary.batss}
\alias{print.summary.batss}
\title{Print function for objects of class 'summary.batss'}
\usage{
\method{print}{summary.batss}(x, ...)
}
\arguments{
\item{x}{An object of class 'summary.batss' (i.e., output of the function \link{summary} used on an output of the function \link{batss.glm}).}

\item{...}{Additional arguments affecting the summary produced.}
}
\value{
Prints a summary for objects of class 'batss'.
}
\description{
Print function for objects of class 'summary.batss'
}
\seealso{
\code{\link[=batss.glm]{batss.glm()}}, the function generating S3 objects of class 'batss'.
}
